@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2020 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
+** Copyright (C) 2021 The Qt Company Ltd.
+** Copyright (C) 2021 Intel Corporation.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
@@ -45,31 +45,25 @@
 #include "qsettings.h"
 #endif
 #include "qlibraryinfo.h"
+#include "qlibraryinfo_p.h"
 #include "qscopedpointer.h"
 
-#ifdef QT_BUILD_QMAKE
-QT_BEGIN_NAMESPACE
-extern QString qmake_libraryInfoFile();
-QT_END_NAMESPACE
-#else
-# include "qcoreapplication.h"
-#endif
+#include "qcoreapplication.h"
 
 #include "private/qglobal_p.h"
+#include "archdetect.cpp"
 #include "qconfig.cpp"
 
 #ifdef Q_OS_DARWIN
 #  include "private/qcore_mac_p.h"
 #endif // Q_OS_DARWIN
 
-#include "archdetect.cpp"
-
-#if !defined(QT_BUILD_QMAKE) && QT_CONFIG(relocatable) && QT_CONFIG(dlopen) && !QT_CONFIG(framework)
-#  include <dlfcn.h>
+#if QT_CONFIG(relocatable) && QT_CONFIG(dlopen) && !QT_CONFIG(framework)
+#    include <dlfcn.h>
 #endif
 
-#if !defined(QT_BUILD_QMAKE) && QT_CONFIG(relocatable) && defined(Q_OS_WIN)
-#  include <qt_windows.h>
+#if QT_CONFIG(relocatable) && defined(Q_OS_WIN)
+#    include <qt_windows.h>
 #endif
 
 QT_BEGIN_NAMESPACE
@@ -78,113 +72,60 @@ extern void qDumpCPUFeatures(); // in qsimd.cpp
 
 #if QT_CONFIG(settings)
 
+static QSettings *findConfiguration();
+
 struct QLibrarySettings
 {
     QLibrarySettings();
     void load();
+    bool havePaths();
+    QSettings *configuration();
 
     QScopedPointer<QSettings> settings;
-#ifdef QT_BUILD_QMAKE
-    bool haveDevicePaths;
-    bool haveEffectiveSourcePaths;
-    bool haveEffectivePaths;
-    bool havePaths;
-#else
+    bool paths;
     bool reloadOnQAppAvailable;
-#endif
 };
 Q_GLOBAL_STATIC(QLibrarySettings, qt_library_settings)
 
-class QLibraryInfoPrivate
-{
-public:
-    static QSettings *findConfiguration();
-#ifdef QT_BUILD_QMAKE
-    static void reload()
-    {
-        if (qt_library_settings.exists())
-            qt_library_settings->load();
-    }
-    static bool haveGroup(QLibraryInfo::PathGroup group)
-    {
-        QLibrarySettings *ls = qt_library_settings();
-        return ls ? (group == QLibraryInfo::EffectiveSourcePaths
-                     ? ls->haveEffectiveSourcePaths
-                     : group == QLibraryInfo::EffectivePaths
-                       ? ls->haveEffectivePaths
-                       : group == QLibraryInfo::DevicePaths
-                         ? ls->haveDevicePaths
-                         : ls->havePaths) : false;
-    }
-#endif
-    static QSettings *configuration()
-    {
-        QLibrarySettings *ls = qt_library_settings();
-        if (ls) {
-#ifndef QT_BUILD_QMAKE
-            if (ls->reloadOnQAppAvailable && QCoreApplication::instance() != nullptr)
-                ls->load();
-#endif
-            return ls->settings.data();
-        } else {
-            return nullptr;
-        }
-    }
-};
-
-static const char platformsSection[] = "Platforms";
-
-QLibrarySettings::QLibrarySettings()
+QLibrarySettings::QLibrarySettings() : paths(false), reloadOnQAppAvailable(false)
 {
     load();
 }
 
+QSettings *QLibrarySettings::configuration()
+{
+    if (reloadOnQAppAvailable && QCoreApplication::instance() != nullptr)
+        load();
+    return settings.data();
+}
+
+bool QLibrarySettings::havePaths()
+{
+    if (reloadOnQAppAvailable && QCoreApplication::instance() != nullptr)
+        load();
+    return paths;
+}
+
 void QLibrarySettings::load()
 {
     // If we get any settings here, those won't change when the application shows up.
-    settings.reset(QLibraryInfoPrivate::findConfiguration());
-#ifndef QT_BUILD_QMAKE
+    settings.reset(findConfiguration());
     reloadOnQAppAvailable = (settings.data() == nullptr && QCoreApplication::instance() == nullptr);
-    bool haveDevicePaths;
-    bool haveEffectivePaths;
-    bool havePaths;
-#endif
+
     if (settings) {
         // This code needs to be in the regular library, as otherwise a qt.conf that
         // works for qmake would break things for dynamically built Qt tools.
         QStringList children = settings->childGroups();
-        haveDevicePaths = children.contains(QLatin1String("DevicePaths"));
-#ifdef QT_BUILD_QMAKE
-        haveEffectiveSourcePaths = children.contains(QLatin1String("EffectiveSourcePaths"));
-#else
-        // EffectiveSourcePaths is for the Qt build only, so needs no backwards compat trickery.
-        bool haveEffectiveSourcePaths = false;
-#endif
-        haveEffectivePaths = haveEffectiveSourcePaths || children.contains(QLatin1String("EffectivePaths"));
-        // Backwards compat: an existing but empty file is claimed to contain the Paths section.
-        havePaths = (!haveDevicePaths && !haveEffectivePaths
-                     && !children.contains(QLatin1String(platformsSection)))
-                    || children.contains(QLatin1String("Paths"));
-#ifndef QT_BUILD_QMAKE
-        if (!havePaths)
-            settings.reset(nullptr);
-#else
-    } else {
-        haveDevicePaths = false;
-        haveEffectiveSourcePaths = false;
-        haveEffectivePaths = false;
-        havePaths = false;
-#endif
+        paths = !children.contains(QLatin1String("Platforms"))
+                || children.contains(QLatin1String("Paths"));
     }
 }
 
-QSettings *QLibraryInfoPrivate::findConfiguration()
+static QSettings *findConfiguration()
 {
-#ifdef QT_BUILD_QMAKE
-    QString qtconfig = qmake_libraryInfoFile();
-    if (!qtconfig.isEmpty())
-        return new QSettings(qtconfig, QSettings::IniFormat);
-#else
+    if (QLibraryInfoPrivate::qtconfManualPath)
+        return new QSettings(*QLibraryInfoPrivate::qtconfManualPath, QSettings::IniFormat);
+
     QString qtconfig = QStringLiteral(":/qt/etc/qt.conf");
     if (QFile::exists(qtconfig))
         return new QSettings(qtconfig, QSettings::IniFormat);
@@ -212,10 +153,28 @@ QSettings *QLibraryInfoPrivate::findConfiguration()
         if (QFile::exists(qtconfig))
             return new QSettings(qtconfig, QSettings::IniFormat);
     }
-#endif
     return nullptr;     //no luck
 }
 
+const QString *QLibraryInfoPrivate::qtconfManualPath = nullptr;
+
+QSettings *QLibraryInfoPrivate::configuration()
+{
+    QLibrarySettings *ls = qt_library_settings();
+    return ls ? ls->configuration() : nullptr;
+}
+
+void QLibraryInfoPrivate::reload()
+{
+    if (qt_library_settings.exists())
+        qt_library_settings->load();
+}
+
+static bool havePaths() {
+    QLibrarySettings *ls = qt_library_settings();
+    return ls && ls->havePaths();
+}
+
 #endif // settings
 
 /*!
@@ -236,8 +195,6 @@ QSettings *QLibraryInfoPrivate::findConfiguration()
     \sa QSysInfo, {Using qt.conf}
 */
 
-#ifndef QT_BUILD_QMAKE
-
 /*!
     \internal
 
@@ -343,7 +300,6 @@ QLibraryInfo::isDebugBuild()
 #endif
 }
 
-#ifndef QT_BOOTSTRAPPED
 /*!
     \since 5.8
     Returns the version of the Qt library.
@@ -354,87 +310,7 @@ QVersionNumber QLibraryInfo::version() noexcept
 {
     return QVersionNumber(QT_VERSION_MAJOR, QT_VERSION_MINOR, QT_VERSION_PATCH);
 }
-#endif // QT_BOOTSTRAPPED
 
-#endif // QT_BUILD_QMAKE
-
-/*
- * To add a new entry in QLibrary::LibraryPath, add it to the enum above the bootstrapped values and:
- * - add its relative path in the qtConfEntries[] array below
- *   (the key is what appears in a qt.conf file)
- * - add a property name in qmake/property.cpp propList[] array
- *   (it's used with qmake -query)
- * - add to qt_config.prf, qt_module.prf, qt_module_fwdpri.prf
- */
-
-#ifdef QT_BUILD_QMAKE
-constexpr size_t qtConfEntriesKeySize = 23;
-#else
-constexpr size_t qtConfEntriesKeySize = 19;
-#endif
-
-static const struct {
-    char key[qtConfEntriesKeySize], value[13];
-} qtConfEntries[] = {
-    { "Prefix", "." },
-    { "Documentation", "doc" }, // should be ${Data}/doc
-    { "Headers", "include" },
-    { "Libraries", "lib" },
-#ifdef Q_OS_WIN
-    { "LibraryExecutables", "bin" },
-#else
-    { "LibraryExecutables", "libexec" }, // should be ${ArchData}/libexec
-#endif
-    { "Binaries", "bin" },
-    { "Plugins", "plugins" }, // should be ${ArchData}/plugins
-    { "Qml2Imports", "qml" }, // should be ${ArchData}/qml
-    { "ArchData", "." },
-    { "Data", "." },
-    { "Translations", "translations" }, // should be ${Data}/translations
-    { "Examples", "examples" },
-    { "Tests", "tests" },
-#ifdef QT_BUILD_QMAKE
-    { "Sysroot", "" },
-    { "SysrootifyPrefix", "" },
-    { "HostBinaries", "bin" },
-#ifdef Q_OS_WIN
-    { "HostLibraryExecutables", "bin" },
-#else
-    { "HostLibraryExecutables", "libexec" },
-#endif
-    { "HostLibraries", "lib" },
-    { "HostData", "." },
-    { "TargetSpec", "" },
-    { "HostSpec", "" },
-    { "HostPrefix", "" },
-#endif
-};
-
-#ifdef QT_BUILD_QMAKE
-void QLibraryInfo::reload()
-{
-    QLibraryInfoPrivate::reload();
-}
-
-void QLibraryInfo::sysrootify(QString *path)
-{
-    if (!QVariant::fromValue(rawLocation(SysrootifyPrefixPath, FinalPaths)).toBool())
-        return;
-
-    const QString sysroot = rawLocation(SysrootPath, FinalPaths);
-    if (sysroot.isEmpty())
-        return;
-
-    if (path->length() > 2 && path->at(1) == QLatin1Char(':')
-        && (path->at(2) == QLatin1Char('/') || path->at(2) == QLatin1Char('\\'))) {
-        path->replace(0, 2, sysroot); // Strip out the drive on Windows targets
-    } else {
-        path->prepend(sysroot);
-    }
-}
-#endif // QT_BUILD_QMAKE
-
-#ifndef QT_BUILD_QMAKE
 static QString prefixFromAppDirHelper()
 {
     QString appDir;
@@ -464,9 +340,8 @@ static QString prefixFromAppDirHelper()
 
     return appDir;
 }
-#endif
 
-#if !defined(QT_BUILD_QMAKE) && QT_CONFIG(relocatable)
+#if QT_CONFIG(relocatable)
 #if !defined(QT_STATIC) && !(defined(Q_OS_DARWIN) && QT_CONFIG(framework)) \
         && (QT_CONFIG(dlopen) || defined(Q_OS_WIN))
 static QString prefixFromQtCoreLibraryHelper(const QString &qtCoreLibraryPath)
@@ -556,7 +431,7 @@ static QString getRelocatablePrefix()
         // executable within the QT_HOST_BIN directory. We're detecting the latter case by checking
         // whether there's an import library corresponding to our QtCore DLL in PREFIX/lib.
         const QString libdir = QString::fromLocal8Bit(
-            qt_configure_strs + qt_configure_str_offsets[QLibraryInfo::LibrariesPath - 1]);
+            qt_configure_strs.viewAt(QLibraryInfo::LibrariesPath - 1));
         const QLatin1Char slash('/');
 #if defined(Q_CC_MINGW)
         const QString implibPrefix = QStringLiteral("lib");
@@ -588,7 +463,7 @@ static QString getRelocatablePrefix()
     // See "Hardware capabilities" in the ld.so documentation and the Qt 5.3.0
     // changelog regarding SSE2 support.
     const QString libdir = QString::fromLocal8Bit(
-        qt_configure_strs + qt_configure_str_offsets[QLibraryInfo::LibrariesPath - 1]);
+        qt_configure_strs.viewAt(QLibraryInfo::LibrariesPath - 1));
     QDir prefixDir(prefixPath);
     while (!prefixDir.exists(libdir)) {
         prefixDir.cdUp();
@@ -606,53 +481,69 @@ static QString getRelocatablePrefix()
 }
 #endif
 
-#if defined(QT_BUILD_QMAKE)
-QString qmake_abslocation();
-
-static QString getPrefixFromHostBinDir(const char *hostBinDirToPrefixPath)
+static QString getPrefix()
 {
-    const QString canonicalQMakePath = QFileInfo(qmake_abslocation()).canonicalPath();
-    return QDir::cleanPath(canonicalQMakePath + QLatin1Char('/')
-                           + QLatin1String(hostBinDirToPrefixPath));
-}
-
-static QString getExtPrefixFromHostBinDir()
-{
-    return getPrefixFromHostBinDir(QT_CONFIGURE_HOSTBINDIR_TO_EXTPREFIX_PATH);
-}
-
-static QString getHostPrefixFromHostBinDir()
-{
-    return getPrefixFromHostBinDir(QT_CONFIGURE_HOSTBINDIR_TO_HOSTPREFIX_PATH);
-}
-#endif
-
-static QString getPrefix(
-#ifdef QT_BUILD_QMAKE
-        QLibraryInfo::PathGroup group
-#endif
-        )
-{
-#if defined(QT_BUILD_QMAKE)
-#  if QT_CONFIGURE_CROSSBUILD
-    if (group == QLibraryInfo::DevicePaths)
-        return QString::fromLocal8Bit(QT_CONFIGURE_PREFIX_PATH);
-#  else
-    Q_UNUSED(group);
-#  endif
-    return getExtPrefixFromHostBinDir();
-#elif QT_CONFIG(relocatable)
+#if QT_CONFIG(relocatable)
     return getRelocatablePrefix();
 #else
     return QString::fromLocal8Bit(QT_CONFIGURE_PREFIX_PATH);
 #endif
 }
 
+QLibraryInfoPrivate::LocationInfo QLibraryInfoPrivate::locationInfo(QLibraryInfo::LibraryPath loc)
+{
+    /*
+     * To add a new entry in QLibraryInfo::LibraryPath, add it to the enum
+     * in qtbase/src/corelib/global/qlibraryinfo.h and:
+     * - add its relative path in the qtConfEntries[] array below
+     *   (the key is what appears in a qt.conf file)
+     */
+    static constexpr auto qtConfEntries = qOffsetStringArray(
+        "Prefix", ".",
+        "Documentation", "doc", // should be ${Data}/doc
+        "Headers", "include",
+        "Libraries", "lib",
+#ifdef Q_OS_WIN
+        "LibraryExecutables", "bin",
+#else
+        "LibraryExecutables", "libexec", // should be ${ArchData}/libexec
+#endif
+        "Binaries", "bin",
+        "Plugins", "plugins", // should be ${ArchData}/plugins
+
+        "QmlImports", "qml", // should be ${ArchData}/qml
+
+        "ArchData", ".",
+        "Data", ".",
+        "Translations", "translations", // should be ${Data}/translations
+        "Examples", "examples",
+        "Tests", "tests"
+    );
+    static constexpr QByteArrayView dot = qtConfEntries.viewAt(1);
+    static_assert(dot.size() == 1);
+    static_assert(dot[0] == '.');
+
+    LocationInfo result;
+
+    if (int(loc) < qtConfEntries.count()) {
+        result.key = QLatin1String(qtConfEntries.viewAt(loc * 2));
+        result.defaultValue = QLatin1String(qtConfEntries.viewAt(loc * 2 + 1));
+        if (result.key == u"QmlImports")
+            result.fallbackKey = u"Qml2Imports"_qs;
+#ifndef Q_OS_WIN // On Windows we use the registry
+    } else if (loc == QLibraryInfo::SettingsPath) {
+        result.key = QLatin1String("Settings");
+        result.defaultValue = QLatin1String(dot);
+#endif
+    }
+
+    return result;
+}
+
 /*! \fn QString QLibraryInfo::location(LibraryLocation loc)
-    \obsolete Use path() instead.
+    \deprecated [6.0] Use path() instead.
 
     Returns the path specified by \a loc.
-    \sa path()
 */
 
 /*!
@@ -662,79 +553,26 @@ static QString getPrefix(
 QString QLibraryInfo::path(LibraryPath p)
 {
     const LibraryPath loc = p;
-#ifdef QT_BUILD_QMAKE // ends inside rawLocation !
-    QString ret = rawLocation(loc, FinalPaths);
-
-    // Automatically prepend the sysroot to target paths
-    if (loc < SysrootPath || loc > LastHostPath)
-        sysrootify(&ret);
-
-    return ret;
-}
-
-QString
-QLibraryInfo::rawLocation(LibraryPath loc, PathGroup group)
-{
-#endif // QT_BUILD_QMAKE, started inside path!
     QString ret;
     bool fromConf = false;
 #if QT_CONFIG(settings)
-#ifdef QT_BUILD_QMAKE
-    // Logic for choosing the right data source: if EffectivePaths are requested
-    // and qt.conf with that section is present, use it, otherwise fall back to
-    // FinalPaths. For FinalPaths, use qt.conf if present and contains not only
-    // [EffectivePaths], otherwise fall back to builtins.
-    // EffectiveSourcePaths falls back to EffectivePaths.
-    // DevicePaths falls back to FinalPaths.
-    PathGroup orig_group = group;
-    if (QLibraryInfoPrivate::haveGroup(group)
-        || (group == EffectiveSourcePaths
-            && (group = EffectivePaths, QLibraryInfoPrivate::haveGroup(group)))
-        || ((group == EffectivePaths || group == DevicePaths)
-            && (group = FinalPaths, QLibraryInfoPrivate::haveGroup(group)))
-        || (group = orig_group, false))
-#else
-    if (QLibraryInfoPrivate::configuration())
-#endif
-    {
+    if (havePaths()) {
         fromConf = true;
 
-        QString key;
-        QString defaultValue;
-        if (unsigned(loc) < sizeof(qtConfEntries)/sizeof(qtConfEntries[0])) {
-            key = QLatin1String(qtConfEntries[loc].key);
-            defaultValue = QLatin1String(qtConfEntries[loc].value);
-        }
-#ifndef Q_OS_WIN // On Windows we use the registry
-        else if (loc == SettingsPath) {
-            key = QLatin1String("Settings");
-            defaultValue = QLatin1String(".");
-        }
-#endif
-
-        if (!key.isNull()) {
+        auto li = QLibraryInfoPrivate::locationInfo(loc);
+        if (!li.key.isNull()) {
             QSettings *config = QLibraryInfoPrivate::configuration();
-            config->beginGroup(QLatin1String(
-#ifdef QT_BUILD_QMAKE
-                   group == DevicePaths ? "DevicePaths" :
-                   group == EffectiveSourcePaths ? "EffectiveSourcePaths" :
-                   group == EffectivePaths ? "EffectivePaths" :
-#endif
-                                             "Paths"));
+            Q_ASSERT(config != nullptr);
+            config->beginGroup(QLatin1String("Paths"));
 
-            ret = config->value(key, defaultValue).toString();
-
-#ifdef QT_BUILD_QMAKE
-            if (ret.isEmpty()) {
-                if (loc == HostPrefixPath)
-                    ret = config->value(QLatin1String(qtConfEntries[PrefixPath].key),
-                                        QLatin1String(qtConfEntries[PrefixPath].value)).toString();
-                else if (loc == TargetSpecPath || loc == HostSpecPath || loc == SysrootifyPrefixPath)
-                    fromConf = false;
-                // The last case here is SysrootPath, which can be legitimately empty.
-                // All other keys have non-empty fallbacks to start with.
+            if (li.fallbackKey.isNull()) {
+                ret = config->value(li.key, li.defaultValue).toString();
+            } else {
+                QVariant v = config->value(li.key);
+                if (!v.isValid())
+                    v = config->value(li.fallbackKey, li.defaultValue);
+                ret = v.toString();
             }
-#endif
 
             int startIndex = 0;
             forever {
@@ -764,66 +602,30 @@ QLibraryInfo::rawLocation(LibraryPath loc, PathGroup group)
 #endif // settings
 
     if (!fromConf) {
-        // "volatile" here is a hack to prevent compilers from doing a
-        // compile-time strlen() on "path". The issue is that Qt installers
-        // will binary-patch the Qt installation paths -- in such scenarios, Qt
-        // will be built with a dummy path, thus the compile-time result of
-        // strlen is meaningless.
-        const char * volatile path = nullptr;
         if (loc == PrefixPath) {
-            ret = getPrefix(
-#ifdef QT_BUILD_QMAKE
-                        group
-#endif
-                   );
-        } else if (unsigned(loc) <= sizeof(qt_configure_str_offsets)/sizeof(qt_configure_str_offsets[0])) {
-            path = qt_configure_strs + qt_configure_str_offsets[loc - 1];
+            ret = getPrefix();
+        } else if (int(loc) <= qt_configure_strs.count()) {
+            ret = QString::fromLocal8Bit(qt_configure_strs.viewAt(loc - 1));
 #ifndef Q_OS_WIN // On Windows we use the registry
         } else if (loc == SettingsPath) {
-            path = QT_CONFIGURE_SETTINGS_PATH;
-#endif
-# ifdef QT_BUILD_QMAKE
-        } else if (loc == HostPrefixPath) {
-            static const QByteArray hostPrefixPath = getHostPrefixFromHostBinDir().toLatin1();
-            path = hostPrefixPath.constData();
-# endif
-        }
-
-        if (path)
+            // Use of volatile is a hack to discourage compilers from calling
+            // strlen(), in the inlined fromLocal8Bit(const char *)'s body, at
+            // compile-time, as Qt installers binary-patch the path, replacing
+            // the dummy path seen at compile-time, typically changing length.
+            const char *volatile path = QT_CONFIGURE_SETTINGS_PATH;
             ret = QString::fromLocal8Bit(path);
-    }
-
-#ifdef QT_BUILD_QMAKE
-    // These values aren't actually paths and thus need to be returned verbatim.
-    if (loc == TargetSpecPath || loc == HostSpecPath || loc == SysrootifyPrefixPath)
-        return ret;
 #endif
+        }
+    }
 
     if (!ret.isEmpty() && QDir::isRelativePath(ret)) {
         QString baseDir;
-#ifdef QT_BUILD_QMAKE
-        if (loc == HostPrefixPath || loc == PrefixPath || loc == SysrootPath) {
-            // We make the prefix/sysroot path absolute to the executable's directory.
-            // loc == PrefixPath while a sysroot is set would make no sense here.
-            // loc == SysrootPath only makes sense if qmake lives inside the sysroot itself.
-            baseDir = QFileInfo(qmake_libraryInfoFile()).absolutePath();
-        } else if (loc > SysrootPath && loc <= LastHostPath) {
-            // We make any other host path absolute to the host prefix directory.
-            baseDir = rawLocation(HostPrefixPath, group);
-        } else {
-            // we make any other path absolute to the prefix directory
-            baseDir = rawLocation(PrefixPath, group);
-            if (group == EffectivePaths)
-                sysrootify(&baseDir);
-        }
-#else
         if (loc == PrefixPath) {
             baseDir = prefixFromAppDirHelper();
         } else {
             // we make any other path absolute to the prefix directory
             baseDir = path(PrefixPath);
         }
-#endif // QT_BUILD_QMAKE
         ret = QDir::cleanPath(baseDir + QLatin1Char('/') + ret);
     }
     return ret;
@@ -844,10 +646,10 @@ QLibraryInfo::rawLocation(LibraryPath loc, PathGroup group)
 
 QStringList QLibraryInfo::platformPluginArguments(const QString &platformName)
 {
-#if !defined(QT_BUILD_QMAKE) && QT_CONFIG(settings)
-    QScopedPointer<const QSettings> settings(QLibraryInfoPrivate::findConfiguration());
+#if QT_CONFIG(settings)
+    QScopedPointer<const QSettings> settings(findConfiguration());
     if (!settings.isNull()) {
-        const QString key = QLatin1String(platformsSection)
+        const QString key = QLatin1String("Platforms")
                 + QLatin1Char('/')
                 + platformName
                 + QLatin1String("Arguments");
@@ -855,7 +657,7 @@ QStringList QLibraryInfo::platformPluginArguments(const QString &platformName)
     }
 #else
     Q_UNUSED(platformName);
-#endif // !QT_BUILD_QMAKE && settings
+#endif // settings
     return QStringList();
 }
 
@@ -873,7 +675,8 @@ QStringList QLibraryInfo::platformPluginArguments(const QString &platformName)
     \value LibraryExecutablesPath The path to installed executables required by libraries at runtime.
     \value BinariesPath The path to installed Qt binaries (tools and applications).
     \value PluginsPath The path to installed Qt plugins.
-    \value Qml2ImportsPath The path to installed QML extensions to import (QML 2.x).
+    \value QmlImportsPath The path to installed QML extensions to import.
+    \value Qml2ImportsPath This value is deprecated. Use QmlImportsPath instead.
     \value ArchDataPath The path to general architecture-dependent Qt data.
     \value DataPath The path to general architecture-independent Qt data.
     \value TranslationsPath The path to translation information for Qt strings.
@@ -886,21 +689,52 @@ QStringList QLibraryInfo::platformPluginArguments(const QString &platformName)
 
 /*!
     \typealias QLibraryInfo::LibraryLocation
-    \obsolete Use LibraryPath with QLibraryInfo::path() instead.
+    \deprecated Use LibraryPath with QLibraryInfo::path() instead.
 */
 
 QT_END_NAMESPACE
 
 #if defined(Q_CC_GNU) && defined(ELF_INTERPRETER)
+#  include <elf.h>
 #  include <stdio.h>
 #  include <stdlib.h>
 
 #include "private/qcoreapplication_p.h"
 
+QT_WARNING_DISABLE_GCC("-Wformat-overflow")
 QT_WARNING_DISABLE_GCC("-Wattributes")
 QT_WARNING_DISABLE_CLANG("-Wattributes")
 QT_WARNING_DISABLE_INTEL(2621)
 
+#  if defined(Q_OS_LINUX)
+#    include "minimum-linux_p.h"
+#  endif
+#  ifdef QT_ELF_NOTE_OS_TYPE
+struct ElfNoteAbiTag
+{
+    static_assert(sizeof(Elf32_Nhdr) == sizeof(Elf64_Nhdr),
+        "The size of an ELF note is wrong (should be 12 bytes)");
+    struct Payload {
+        Elf32_Word ostype = QT_ELF_NOTE_OS_TYPE;
+        Elf32_Word major = QT_ELF_NOTE_OS_MAJOR;
+        Elf32_Word minor = QT_ELF_NOTE_OS_MINOR;
+#    ifdef QT_ELF_NOTE_OS_PATCH
+        Elf32_Word patch = QT_ELF_NOTE_OS_PATCH;
+#    endif
+    };
+
+    Elf32_Nhdr header = {
+        .n_namesz = sizeof(name),
+        .n_descsz = sizeof(Payload),
+        .n_type = NT_GNU_ABI_TAG
+    };
+    char name[sizeof ELF_NOTE_GNU] = ELF_NOTE_GNU;  // yes, include the null terminator
+    Payload payload = {};
+};
+__attribute__((section(".note.ABI-tag"), aligned(4), used))
+extern constexpr ElfNoteAbiTag QT_MANGLE_NAMESPACE(qt_abi_tag) = {};
+#  endif
+
 extern const char qt_core_interpreter[] __attribute__((section(".interp")))
     = ELF_INTERPRETER;
 
@@ -914,9 +748,9 @@ void qt_core_boilerplate()
            "Installation prefix: %s\n"
            "Library path:        %s\n"
            "Plugin path:         %s\n",
-           qt_configure_prefix_path_str + 12,
-           qt_configure_strs + qt_configure_str_offsets[QT_PREPEND_NAMESPACE(QLibraryInfo)::LibrariesPath - 1],
-           qt_configure_strs + qt_configure_str_offsets[QT_PREPEND_NAMESPACE(QLibraryInfo)::PluginsPath - 1]);
+           QT_CONFIGURE_PREFIX_PATH,
+           qt_configure_strs[QT_PREPEND_NAMESPACE(QLibraryInfo)::LibrariesPath - 1],
+           qt_configure_strs[QT_PREPEND_NAMESPACE(QLibraryInfo)::PluginsPath - 1]);
 
     QT_PREPEND_NAMESPACE(qDumpCPUFeatures)();
 
