@@ -39,8 +39,6 @@
 ****************************************************************************/
 
 #include "qsemaphore.h"
-
-#ifndef QT_NO_THREAD
 #include "qmutex.h"
 #include "qfutex_p.h"
 #include "qwaitcondition.h"
@@ -129,13 +127,13 @@ using namespace QtFutex;
  */
 
 #if defined(FUTEX_OP) && QT_POINTER_SIZE > 4
-static Q_CONSTEXPR bool futexHasWaiterCount = true;
+static constexpr bool futexHasWaiterCount = true;
 #else
-static Q_CONSTEXPR bool futexHasWaiterCount = false;
+static constexpr bool futexHasWaiterCount = false;
 #endif
 
-static const quintptr futexNeedsWakeAllBit =
-        Q_UINT64_C(1) << (sizeof(quintptr) * CHAR_BIT - 1);
+static constexpr quintptr futexNeedsWakeAllBit = futexHasWaiterCount ?
+        (Q_UINT64_C(1) << (sizeof(quintptr) * CHAR_BIT - 1)) : 0x80000000U;
 
 static int futexAvailCounter(quintptr v)
 {
@@ -152,10 +150,11 @@ static int futexAvailCounter(quintptr v)
 
 static bool futexNeedsWake(quintptr v)
 {
-    // If we're counting waiters, the number of waiters is stored in the low 31
-    // bits of the high word (that is, bits 32-62). If we're not, then we use
-    // bit 31 to indicate anyone is waiting. Either way, if any bit 31 or above
-    // is set, there are waiters.
+    // If we're counting waiters, the number of waiters plus value is stored in the
+    // low 31 bits of the high word (that is, bits 32-62). If we're not, then we only
+    // use futexNeedsWakeAllBit to indicate anyone is waiting.
+    if constexpr (futexHasWaiterCount)
+        return unsigned(quint64(v) >> 32) > unsigned(v);
     return v >> 31;
 }
 
@@ -170,6 +169,7 @@ static QBasicAtomicInteger<quint32> *futexLow32(QBasicAtomicInteger<quintptr> *p
 
 static QBasicAtomicInteger<quint32> *futexHigh32(QBasicAtomicInteger<quintptr> *ptr)
 {
+    Q_ASSERT(futexHasWaiterCount);
     auto result = reinterpret_cast<QBasicAtomicInteger<quint32> *>(ptr);
 #if Q_BYTE_ORDER == Q_LITTLE_ENDIAN && QT_POINTER_SIZE > 4
     ++result;
@@ -185,31 +185,16 @@ futexSemaphoreTryAcquire_loop(QBasicAtomicInteger<quintptr> &u, quintptr curValu
     int n = int(unsigned(nn));
 
     // we're called after one testAndSet, so start by waiting first
-    goto start_wait;
-
-    forever {
-        if (futexAvailCounter(curValue) >= n) {
-            // try to acquire
-            quintptr newValue = curValue - nn;
-            if (u.testAndSetOrdered(curValue, newValue, curValue))
-                return true;        // succeeded!
-            continue;
-        }
-
-        // not enough tokens available, put us to wait
-        if (remainingTime == 0)
-            return false;
-
+    for (;;) {
         // indicate we're waiting
-start_wait:
         auto ptr = futexLow32(&u);
         if (n > 1 || !futexHasWaiterCount) {
             u.fetchAndOrRelaxed(futexNeedsWakeAllBit);
             curValue |= futexNeedsWakeAllBit;
-            if (n > 1 && futexHasWaiterCount) {
+            if constexpr (futexHasWaiterCount) {
+                Q_ASSERT(n > 1);
                 ptr = futexHigh32(&u);
-                //curValue >>= 32;  // but this is UB in 32-bit, so roundabout:
-                curValue = quint64(curValue) >> 32;
+                curValue >>= 32;
             }
         }
 
@@ -224,6 +209,17 @@ start_wait:
         curValue = u.loadAcquire();
         if (IsTimed)
             remainingTime = timer.remainingTimeNSecs();
+
+        // try to acquire
+        while (futexAvailCounter(curValue) >= n) {
+            quintptr newValue = curValue - nn;
+            if (u.testAndSetOrdered(curValue, newValue, curValue))
+                return true;        // succeeded!
+        }
+
+        // not enough tokens available, put us to wait
+        if (remainingTime == 0)
+            return false;
     }
 }
 
@@ -246,15 +242,17 @@ template <bool IsTimed> bool futexSemaphoreTryAcquire(QBasicAtomicInteger<quintp
         return false;
 
     // we need to wait
-    quintptr oneWaiter = quintptr(Q_UINT64_C(1) << 32); // zero on 32-bit
-    if (futexHasWaiterCount) {
-        // increase the waiter count
-        u.fetchAndAddRelaxed(oneWaiter);
-
+    constexpr quintptr oneWaiter = quintptr(Q_UINT64_C(1) << 32); // zero on 32-bit
+    if constexpr (futexHasWaiterCount) {
         // We don't use the fetched value from above so futexWait() fails if
         // it changed after the testAndSetOrdered above.
-        if ((quint64(curValue) >> 32) == 0x7fffffff)
-            return false;       // overflow!
+        if (((curValue >> 32) & 0x7fffffffU) == 0x7fffffffU) {
+            qCritical() << "Waiter count overflow in QSemaphore";
+            return false;
+        }
+
+        // increase the waiter count
+        u.fetchAndAddRelaxed(oneWaiter);
         curValue += oneWaiter;
 
         // Also adjust nn to subtract oneWaiter when we succeed in acquiring.
@@ -264,9 +262,11 @@ template <bool IsTimed> bool futexSemaphoreTryAcquire(QBasicAtomicInteger<quintp
     if (futexSemaphoreTryAcquire_loop<IsTimed>(u, curValue, nn, timeout))
         return true;
 
+    Q_ASSERT(IsTimed);
+
     if (futexHasWaiterCount) {
         // decrement the number of threads waiting
-        Q_ASSERT(futexHigh32(&u)->load() & 0x7fffffffU);
+        Q_ASSERT(futexHigh32(&u)->loadRelaxed() & 0x7fffffffU);
         u.fetchAndSubRelaxed(oneWaiter);
     }
     return false;
@@ -295,7 +295,7 @@ QSemaphore::QSemaphore(int n)
         quintptr nn = unsigned(n);
         if (futexHasWaiterCount)
             nn |= quint64(nn) << 32;    // token count replicated in high word
-        u.store(nn);
+        u.storeRelaxed(nn);
     } else {
         d = new QSemaphorePrivate(n);
     }
@@ -359,47 +359,31 @@ void QSemaphore::release(int n)
         quintptr prevValue = u.fetchAndAddRelease(nn);
         if (futexNeedsWake(prevValue)) {
 #ifdef FUTEX_OP
-            if (!futexHasWaiterCount) {
-                /*
-                   On 32-bit systems, all waiters are waiting on the same address,
-                   so we'll wake them all and ask the kernel to clear the high bit.
-
-                   atomic {
-                      int oldval = u;
-                      u = oldval & ~(1 << 31);
-                      futexWake(u, INT_MAX);
-                      if (oldval == 0)       // impossible condition
-                          futexWake(u, INT_MAX);
-                   }
-                */
-                quint32 op = FUTEX_OP_ANDN | FUTEX_OP_OPARG_SHIFT;
-                quint32 oparg = 31;
-                quint32 cmp = FUTEX_OP_CMP_EQ;
-                quint32 cmparg = 0;
-                futexWakeOp(u, INT_MAX, INT_MAX, u, FUTEX_OP(op, oparg, cmp, cmparg));
-            } else {
+            if (futexHasWaiterCount) {
                 /*
                    On 64-bit systems, the single-token waiters wait on the low half
                    and the multi-token waiters wait on the upper half. So we ask
                    the kernel to wake up n single-token waiters and all multi-token
-                   waiters (if any), then clear the multi-token wait bit.
+                   waiters (if any), and clear the multi-token wait bit.
 
                    atomic {
                       int oldval = *upper;
-                      *upper = oldval & ~(1 << 31);
+                      *upper = oldval | 0;
                       futexWake(lower, n);
-                      if (oldval < 0)   // sign bit set
+                      if (oldval != 0)   // always true
                           futexWake(upper, INT_MAX);
                    }
                 */
-                quint32 op = FUTEX_OP_ANDN | FUTEX_OP_OPARG_SHIFT;
-                quint32 oparg = 31;
-                quint32 cmp = FUTEX_OP_CMP_LT;
+                quint32 op = FUTEX_OP_OR;
+                quint32 oparg = 0;
+                quint32 cmp = FUTEX_OP_CMP_NE;
                 quint32 cmparg = 0;
+                u.fetchAndAndRelease(futexNeedsWakeAllBit - 1);
                 futexWakeOp(*futexLow32(&u), n, INT_MAX, *futexHigh32(&u), FUTEX_OP(op, oparg, cmp, cmparg));
+                return;
             }
-#else
-            // Unset the bit and wake everyone. There are two possibibilies
+#endif
+            // Unset the bit and wake everyone. There are two possibilities
             // under which a thread can set the bit between the AND and the
             // futexWake:
             // 1) it did see the new counter value, but it wasn't enough for
@@ -407,8 +391,12 @@ void QSemaphore::release(int n)
             // 2) it did not see the new counter value, in which case its
             //    futexWait will fail.
             u.fetchAndAndRelease(futexNeedsWakeAllBit - 1);
-            futexWakeAll(u);
-#endif
+            if (futexHasWaiterCount) {
+                futexWakeAll(*futexLow32(&u));
+                futexWakeAll(*futexHigh32(&u));
+            } else {
+                futexWakeAll(u);
+            }
         }
         return;
     }
@@ -427,7 +415,7 @@ void QSemaphore::release(int n)
 int QSemaphore::available() const
 {
     if (futexAvailable())
-        return futexAvailCounter(u.load());
+        return futexAvailCounter(u.loadRelaxed());
 
     QMutexLocker locker(&d->mutex);
     return d->avail;
@@ -487,20 +475,61 @@ bool QSemaphore::tryAcquire(int n, int timeout)
 
     QDeadlineTimer timer(timeout);
     QMutexLocker locker(&d->mutex);
-    qint64 remainingTime = timer.remainingTime();
-    while (n > d->avail && remainingTime != 0) {
-        if (!d->cond.wait(locker.mutex(), remainingTime))
+    while (n > d->avail && !timer.hasExpired()) {
+        if (!d->cond.wait(locker.mutex(), timer))
             return false;
-        remainingTime = timer.remainingTime();
     }
     if (n > d->avail)
         return false;
     d->avail -= n;
     return true;
-
-
 }
 
+/*!
+    \fn template <typename Rep, typename Period> QSemaphore::tryAcquire(int n, std::chrono::duration<Rep, Period> timeout)
+    \overload
+    \since 6.3
+*/
+
+/*!
+    \fn bool QSemaphore::try_acquire()
+    \since 6.3
+
+    This function is provided for \c{std::counting_semaphore} compatibility.
+
+    It is equivalent to calling \c{tryAcquire(1)}, where the function returns
+    \c true on acquiring the resource successfully.
+
+    \sa tryAcquire(), try_acquire_for(), try_acquire_until()
+*/
+
+/*!
+    \fn template <typename Rep, typename Period> bool QSemaphore::try_acquire_for(const std::chrono::duration<Rep, Period> &timeout)
+    \since 6.3
+
+    This function is provided for \c{std::counting_semaphore} compatibility.
+
+    It is equivalent to calling \c{tryAcquire(1, timeout)}, where the call
+    times out on the given \a timeout value. The function returns \c true
+    on accquiring the resource successfully.
+
+    \sa tryAcquire(), try_acquire(), try_acquire_until()
+*/
+
+/*!
+    \fn template <typename Clock, typename Duration> bool QSemaphore::try_acquire_until(const std::chrono::time_point<Clock, Duration> &tp)
+    \since 6.3
+
+    This function is provided for \c{std::counting_semaphore} compatibility.
+
+    It is equivalent to calling \c{tryAcquire(1, tp - Clock::now())},
+    which means that the \a tp (time point) is recorded, ignoring the
+    adjustments to \c{Clock} while waiting. The function returns \c true
+    on acquiring the resource successfully.
+
+    \sa tryAcquire(), try_acquire(), try_acquire_for()
+*/
+
 /*!
     \class QSemaphoreReleaser
     \brief The QSemaphoreReleaser class provides exception-safe deferral of a QSemaphore::release() call.
@@ -519,10 +548,7 @@ bool QSemaphore::tryAcquire(int n, int timeout)
     You can use this to reliably release a semaphore to avoid dead-lock
     in the face of exceptions or early returns:
 
-    \code
-    // ... do something that may throw or return early
-    sem.release();
-    \endcode
+    \snippet code/src_corelib_thread_qsemaphore.cpp 4
 
     If an early return is taken or an exception is thrown before the
     \c{sem.release()} call is reached, the semaphore is not released,
@@ -531,11 +557,7 @@ bool QSemaphore::tryAcquire(int n, int timeout)
 
     When using RAII instead:
 
-    \code
-    const QSemaphoreReleaser releaser(sem);
-    // ... do something that may throw or early return
-    // implicitly calls sem.release() here and at every other return in between
-    \endcode
+    \snippet code/src_corelib_thread_qsemaphore.cpp 5
 
     this can no longer happen, because the compiler will make sure that
     the QSemaphoreReleaser destructor is always called, and therefore
@@ -545,17 +567,7 @@ bool QSemaphore::tryAcquire(int n, int timeout)
     from functions to transfer responsibility for releasing a semaphore
     out of a function or a scope:
 
-    \code
-    { // some scope
-        QSemaphoreReleaser releaser; // does nothing
-        // ...
-        if (someCondition) {
-            releaser = QSemaphoreReleaser(sem);
-            // ...
-        }
-        // ...
-    } // conditionally calls sem.release(), depending on someCondition
-    \endcode
+    \snippet code/src_corelib_thread_qsemaphore.cpp 6
 
     A QSemaphoreReleaser can be canceled by a call to cancel(). A canceled
     semaphore releaser will no longer call QSemaphore::release() in its
@@ -615,7 +627,7 @@ bool QSemaphore::tryAcquire(int n, int timeout)
 /*!
     \fn QSemaphoreReleaser::swap(QSemaphoreReleaser &other)
 
-    Exchanges the responsibilites of \c{*this} and \a other.
+    Exchanges the responsibilities of \c{*this} and \a other.
 
     Unlike move assignment, neither of the two objects ever releases its
     semaphore, if any, as a consequence of swapping.
@@ -627,7 +639,7 @@ bool QSemaphore::tryAcquire(int n, int timeout)
     \fn QSemaphoreReleaser::semaphore() const
 
     Returns a pointer to the QSemaphore object provided to the constructor,
-    or by the last move assignment, if any. Otherwise, returns \c nullptr.
+    or by the last move assignment, if any. Otherwise, returns \nullptr.
 */
 
 /*!
@@ -635,18 +647,12 @@ bool QSemaphore::tryAcquire(int n, int timeout)
 
     Cancels this QSemaphoreReleaser such that the destructor will no longer
     call \c{semaphore()->release()}. Returns the value of semaphore()
-    before this call. After this call, semaphore() will return \c nullptr.
+    before this call. After this call, semaphore() will return \nullptr.
 
     To enable again, assign a new QSemaphoreReleaser:
 
-    \code
-    releaser.cancel(); // avoid releasing old semaphore()
-    releaser = QSemaphoreReleaser(sem, 42);
-    // now will call sem.release(42) when 'releaser' is destroyed
-    \endcode
+    \snippet code/src_corelib_thread_qsemaphore.cpp 7
 */
 
 
 QT_END_NAMESPACE
-
-#endif // QT_NO_THREAD
