@@ -142,7 +142,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \fn QIcon QFileSystemModel::fileName(const QModelIndex &index) const
+    \fn QString QFileSystemModel::fileName(const QModelIndex &index) const
 
     Returns the file name for the item stored in the model under the given
     \a index.
@@ -720,6 +720,9 @@ QVariant QFileSystemModel::data(const QModelIndex &index, int role) const
 
     switch (role) {
     case Qt::EditRole:
+        if (index.column() == 0)
+            return d->name(index);
+        Q_FALLTHROUGH();
     case Qt::DisplayRole:
         switch (index.column()) {
         case 0: return d->displayName(index);
@@ -893,7 +896,7 @@ bool QFileSystemModel::setData(const QModelIndex &idx, const QVariant &value, in
             *After re-naming something we don't want the selection to change*
             - can't remove rows and later insert
             - can't quickly remove and insert
-            - index pointer can't change because treeview doesn't use persistant index's
+            - index pointer can't change because treeview doesn't use persistent index's
 
             - if this get any more complicated think of changing it to just
               use layoutChanged
@@ -904,14 +907,14 @@ bool QFileSystemModel::setData(const QModelIndex &idx, const QVariant &value, in
         int visibleLocation = parentNode->visibleLocation(parentNode->children.value(indexNode->fileName)->fileName);
 
         parentNode->visibleChildren.removeAt(visibleLocation);
-        QScopedPointer<QFileSystemModelPrivate::QFileSystemNode> nodeToRename(parentNode->children.take(oldName));
+        std::unique_ptr<QFileSystemModelPrivate::QFileSystemNode> nodeToRename(parentNode->children.take(oldName));
         nodeToRename->fileName = newName;
         nodeToRename->parent = parentNode;
 #if QT_CONFIG(filesystemwatcher)
         nodeToRename->populate(d->fileInfoGatherer.getInfo(QFileInfo(parentPath, newName)));
 #endif
         nodeToRename->isVisible = true;
-        parentNode->children[newName] = nodeToRename.take();
+        parentNode->children[newName] = nodeToRename.release();
         parentNode->visibleChildren.insert(visibleLocation, newName);
 
         d->delayedSort();
@@ -1591,9 +1594,11 @@ void QFileSystemModel::setFilter(QDir::Filters filters)
     Q_D(QFileSystemModel);
     if (d->filters == filters)
         return;
+    const bool changingCaseSensitivity =
+        filters.testFlag(QDir::CaseSensitive) != d->filters.testFlag(QDir::CaseSensitive);
     d->filters = filters;
-    // CaseSensitivity might have changed
-    setNameFilters(nameFilters());
+    if (changingCaseSensitivity)
+        d->rebuildNameFilterRegexps();
     d->forceSort = true;
     d->delayedSort();
 }
@@ -1690,7 +1695,6 @@ bool QFileSystemModel::nameFilterDisables() const
 */
 void QFileSystemModel::setNameFilters(const QStringList &filters)
 {
-    // Prep the regexp's ahead of time
 #if QT_CONFIG(regularexpression)
     Q_D(QFileSystemModel);
 
@@ -1713,6 +1717,7 @@ void QFileSystemModel::setNameFilters(const QStringList &filters)
     }
 
     d->nameFilters = filters;
+    d->rebuildNameFilterRegexps();
     d->forceSort = true;
     d->delayedSort();
 #else
@@ -2149,15 +2154,13 @@ bool QFileSystemModelPrivate::passNameFilters(const QFileSystemNode *node) const
 
     // Check the name regularexpression filters
     if (!(node->isDir() && (filters & QDir::AllDirs))) {
-        auto cs = (filters & QDir::CaseSensitive) ? Qt::CaseSensitive : Qt::CaseInsensitive;
-
-        for (const auto &nameFilter : nameFilters) {
-            auto rx = QRegularExpression::fromWildcard(nameFilter, cs);
-            QRegularExpressionMatch match = rx.match(node->fileName);
-            if (match.hasMatch())
-                return true;
-        }
-        return false;
+        const auto matchesNodeFileName = [node](const QRegularExpression &re)
+        {
+            return node->fileName.contains(re);
+        };
+        return std::any_of(nameFiltersRegexps.begin(),
+                           nameFiltersRegexps.end(),
+                           matchesNodeFileName);
     }
 #else
     Q_UNUSED(node);
@@ -2165,6 +2168,23 @@ bool QFileSystemModelPrivate::passNameFilters(const QFileSystemNode *node) const
     return true;
 }
 
+#if QT_CONFIG(regularexpression)
+void QFileSystemModelPrivate::rebuildNameFilterRegexps()
+{
+    nameFiltersRegexps.clear();
+    nameFiltersRegexps.reserve(nameFilters.size());
+    const auto cs = (filters & QDir::CaseSensitive) ? Qt::CaseSensitive : Qt::CaseInsensitive;
+    const auto convertWildcardToRegexp = [cs](const QString &nameFilter)
+    {
+        return QRegularExpression::fromWildcard(nameFilter, cs);
+    };
+    std::transform(nameFilters.constBegin(),
+                   nameFilters.constEnd(),
+                   std::back_inserter(nameFiltersRegexps),
+                   convertWildcardToRegexp);
+}
+#endif
+
 QT_END_NAMESPACE
 
 #include "moc_qfilesystemmodel.cpp"
