@@ -46,9 +46,7 @@
 #include "qvariant.h"
 #include "qbuffer.h"
 #include "qimage.h"
-#if QT_CONFIG(textcodec)
-#include "qtextcodec.h"
-#endif
+#include "private/qstringconverter_p.h"
 
 #include "private/qguiapplication_p.h"
 #include <qpa/qplatformintegration.h>
@@ -85,7 +83,7 @@ QT_BEGIN_NAMESPACE
 
     A typical example of the use of these functions follows:
 
-    \snippet droparea.cpp 0
+    \snippet droparea/droparea.cpp 0
 
     \section1 Notes for X11 Users
 
@@ -139,17 +137,6 @@ QT_BEGIN_NAMESPACE
 
     \endlist
 
-    \section1 Notes for Universal Windows Platform Users
-
-    \list
-
-    \li The Universal Windows Platform only allows to query the
-    clipboard in case the application is active and an application
-    window has focus. Accessing the clipboard data when in background
-    will fail due to access denial.
-
-    \endlist
-
     \sa QGuiApplication
 */
 
@@ -299,17 +286,10 @@ QString QClipboard::text(QString &subtype, Mode mode) const
     }
 
     const QByteArray rawData = data->data(QLatin1String("text/") + subtype);
-
-#if QT_CONFIG(textcodec)
-    QTextCodec* codec = QTextCodec::codecForMib(106); // utf-8 is default
-    if (subtype == QLatin1String("html"))
-        codec = QTextCodec::codecForHtml(rawData, codec);
-    else
-        codec = QTextCodec::codecForUtfText(rawData, codec);
-    return codec->toUnicode(rawData);
-#else // textcodec
-    return rawData;
-#endif // textcodec
+    auto encoding = QStringConverter::encodingForData(rawData);
+    if (!encoding)
+        encoding = QStringConverter::Utf8;
+    return QStringDecoder(*encoding).decode(rawData);
 }
 
 /*!
@@ -461,7 +441,7 @@ void QClipboard::setPixmap(const QPixmap &pixmap, Mode mode)
 const QMimeData* QClipboard::mimeData(Mode mode) const
 {
     QPlatformClipboard *clipboard = QGuiApplicationPrivate::platformIntegration()->clipboard();
-    if (!clipboard->supportsMode(mode)) return 0;
+    if (!clipboard->supportsMode(mode)) return nullptr;
     return clipboard->mimeData(mode);
 }
 
@@ -488,7 +468,7 @@ void QClipboard::setMimeData(QMimeData* src, Mode mode)
 {
     QPlatformClipboard *clipboard = QGuiApplicationPrivate::platformIntegration()->clipboard();
     if (!clipboard->supportsMode(mode)) {
-        if (src != 0) {
+        if (src != nullptr) {
             qDebug("Data set on unsupported clipboard mode. QMimeData object will be deleted.");
             src->deleteLater();
         }
@@ -512,7 +492,7 @@ void QClipboard::setMimeData(QMimeData* src, Mode mode)
 */
 void QClipboard::clear(Mode mode)
 {
-    setMimeData(0, mode);
+    setMimeData(nullptr, mode);
 }
 
 /*!
@@ -602,9 +582,8 @@ void QClipboard::emitChanged(Mode mode)
         case FindBuffer:
             emit findBufferChanged();
         break;
-        default:
-        break;
     }
+
     emit changed(mode);
 }
 
