@@ -633,7 +633,7 @@ QMatrix4x4& QMatrix4x4::operator/=(float divisor)
     \fn QVector3D operator*(const QVector3D& vector, const QMatrix4x4& matrix)
     \relates QMatrix4x4
 
-    \deprecated Convert the QVector3D to a QVector4D first, then multiply.
+    \deprecated [6.1] Convert the QVector3D to a QVector4D first, then multiply.
 
     Returns the result of transforming \a vector according to \a matrix,
     with the matrix applied post-vector. The vector is transformed as a point.
@@ -643,7 +643,7 @@ QMatrix4x4& QMatrix4x4::operator/=(float divisor)
     \fn QVector3D operator*(const QMatrix4x4& matrix, const QVector3D& vector)
     \relates QMatrix4x4
 
-    \deprecated Use QMatrix4x4::map() or QMatrix4x4::mapVector() instead.
+    \deprecated [6.1] Use QMatrix4x4::map() or QMatrix4x4::mapVector() instead.
 
     Returns the result of transforming \a vector according to \a matrix,
     with the matrix applied pre-vector. The vector is transformed as a
@@ -696,7 +696,7 @@ QMatrix4x4& QMatrix4x4::operator/=(float divisor)
     \fn QPoint operator*(const QMatrix4x4& matrix, const QPoint& point)
     \relates QMatrix4x4
 
-    \deprecated Use QMatrix4x4::map() instead.
+    \deprecated [6.1] Use QMatrix4x4::map() instead.
 
     Returns the result of transforming \a point according to \a matrix,
     with the matrix applied pre-point.
@@ -706,7 +706,7 @@ QMatrix4x4& QMatrix4x4::operator/=(float divisor)
     \fn QPointF operator*(const QMatrix4x4& matrix, const QPointF& point)
     \relates QMatrix4x4
 
-    \deprecated Use QMatrix4x4::map() instead.
+    \deprecated [6.1] Use QMatrix4x4::map() instead.
 
     Returns the result of transforming \a point according to \a matrix,
     with the matrix applied pre-point.
@@ -1617,7 +1617,7 @@ void QMatrix4x4::viewport(float left, float bottom, float width, float height, f
     \deprecated
 
     Flips between right-handed and left-handed coordinate systems
-    by multiplying the y and z co-ordinates by -1.  This is normally
+    by multiplying the y and z coordinates by -1.  This is normally
     used to create a left-handed orthographic view without scaling
     the viewport as ortho() does.
 
@@ -1662,7 +1662,7 @@ void QMatrix4x4::copyDataTo(float *values) const
 
     The returned QTransform is formed by simply dropping the
     third row and third column of the QMatrix4x4.  This is suitable
-    for implementing orthographic projections where the z co-ordinate
+    for implementing orthographic projections where the z coordinate
     should be dropped rather than projected.
 */
 QTransform QMatrix4x4::toTransform() const
@@ -1677,14 +1677,14 @@ QTransform QMatrix4x4::toTransform() const
     corresponds to this matrix.
 
     If \a distanceToPlane is non-zero, it indicates a projection
-    factor to use to adjust for the z co-ordinate.  The value of
+    factor to use to adjust for the z coordinate.  The value of
     1024 corresponds to the projection factor used
     by QTransform::rotate() for the x and y axes.
 
     If \a distanceToPlane is zero, then the returned QTransform
     is formed by simply dropping the third row and third column
     of the QMatrix4x4.  This is suitable for implementing
-    orthographic projections where the z co-ordinate should
+    orthographic projections where the z coordinate should
     be dropped rather than projected.
 */
 QTransform QMatrix4x4::toTransform(float distanceToPlane) const
@@ -1923,15 +1923,15 @@ QMatrix4x4 QMatrix4x4::orthonormalInverse() const
 
     Normally the QMatrix4x4 class keeps track of this special type internally
     as operations are performed.  However, if the matrix is modified
-    directly with \l {QMatrix4x4::}{operator()()} or data(), then
-    QMatrix4x4 will lose track of the special type and will revert to the
-    safest but least efficient operations thereafter.
+    directly with operator()(int, int) or data(), then QMatrix4x4 will
+    lose track of the special type and will revert to the safest but least
+    efficient operations thereafter.
 
     By calling optimize() after directly modifying the matrix,
     the programmer can force QMatrix4x4 to recover the special type if
     the elements appear to conform to one of the known optimized types.
 
-    \sa {QMatrix4x4::}{operator()()}, data(), translate()
+    \sa operator()(int, int), data(), translate()
 */
 void QMatrix4x4::optimize()
 {
