@@ -54,6 +54,12 @@
 
 QT_BEGIN_NAMESPACE
 
+// QColor fits into QVariant's internal storage on 64bit systems.
+// It could also fit on 32bit systems, but we cannot make it happen in Qt6, due to BC.
+#if QT_VERSION >= QT_VERSION_CHECK(7,0,0) || QT_POINTER_SIZE > 4
+static_assert(sizeof(QColor) <= QVariant::Private::MaxInternalSize);
+#endif
+
 /*!
     \internal
     If s[0..n] is a valid hex number, returns its integer value,
@@ -780,7 +786,7 @@ QColor::QColor(QRgba64 rgba64) noexcept
 
     Constructs a color with the given \a spec.
 
-    This function is primarly present to avoid that QColor::Invalid
+    This function is primarily present to avoid that QColor::Invalid
     becomes a valid color by accident.
 */
 
@@ -867,7 +873,6 @@ QString QColor::name(NameFormat format) const
     return QString();
 }
 
-#if QT_STRINGVIEW_LEVEL < 2
 /*!
     Sets the RGB value of this QColor to \a name, which may be in one
     of these formats:
@@ -896,7 +901,6 @@ void QColor::setNamedColor(const QString &name)
 {
     setColorFromString(qToStringViewIgnoringNull(name));
 }
-#endif
 
 /*!
     \overload
@@ -918,7 +922,6 @@ void QColor::setNamedColor(QLatin1String name)
     setColorFromString(name);
 }
 
-#if QT_STRINGVIEW_LEVEL < 2
 /*!
    \since 4.7
 
@@ -934,7 +937,6 @@ bool QColor::isValidColor(const QString &name)
 {
     return isValidColor(qToStringViewIgnoringNull(name));
 }
-#endif
 
 /*!
     \overload
