@@ -62,6 +62,10 @@
 
 #include <qtgui_tracepoints_p.h>
 
+#ifdef Q_OS_WIN
+#include <QtGui/private/qwindowsfontdatabasebase_p.h>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 Q_LOGGING_CATEGORY(lcFontDb, "qt.text.font.db")
@@ -304,12 +308,10 @@ void QFontDatabasePrivate::invalidate()
     emit static_cast<QGuiApplication *>(QCoreApplication::instance())->fontDatabaseChanged();
 }
 
-QtFontFamily *QFontDatabasePrivate::family(const QString &family, FamilyRequestFlags flags)
+QtFontFamily *QFontDatabasePrivate::family(const QString &f, FamilyRequestFlags flags)
 {
     QtFontFamily *fam = nullptr;
 
-    const QString f = family.trimmed();
-
     int low = 0;
     int high = count;
     int pos = count / 2;
@@ -783,7 +785,7 @@ QFontEngine *loadSingleEngine(int script,
         if (style->key.stretch != 0 && request.stretch != 0
             && (request.styleName.isEmpty() || request.styleName != style->styleName)) {
             def.stretch = (request.stretch * 100 + style->key.stretch / 2) / style->key.stretch;
-        } else {
+        } else if (request.stretch == QFont::AnyStretch) {
             def.stretch = 100;
         }
 
@@ -820,7 +822,7 @@ QFontEngine *loadEngine(int script, const QFontDef &request,
     QFontEngine *engine = loadSingleEngine(script, request, family, foundry, style, size);
 
     if (engine && !(request.styleStrategy & QFont::NoFontMerging) && !engine->symbol) {
-        Q_TRACE(QFontDatabase_loadEngine, request.family, request.pointSize);
+        Q_TRACE(QFontDatabase_loadEngine, request.families, request.pointSize);
 
         QPlatformFontDatabase *pfdb = QGuiApplicationPrivate::platformIntegration()->fontDatabase();
         QFontEngineMulti *pfMultiEngine = pfdb->fontEngineMulti(engine, QChar::Script(script));
@@ -1051,9 +1053,13 @@ static bool matchFamilyName(const QString &familyName, QtFontFamily *f)
 
     Tries to find the best match for a given request and family/foundry
 */
-static int match(int script, const QFontDef &request,
-                 const QString &family_name, const QString &foundry_name,
-                 QtFontDesc *desc, const QList<int> &blacklistedFamilies)
+static int match(int script,
+                 const QFontDef &request,
+                 const QString &family_name,
+                 const QString &foundry_name,
+                 QtFontDesc *desc,
+                 const QList<int> &blacklistedFamilies,
+                 unsigned int *resultingScore = nullptr)
 {
     int result = -1;
 
@@ -1068,13 +1074,15 @@ static int match(int script, const QFontDef &request,
     qCDebug(lcFontMatch, "QFontDatabase::match\n"
              "  request:\n"
              "    family: %s [%s], script: %d\n"
+             "    styleName: %s\n"
              "    weight: %d, style: %d\n"
              "    stretch: %d\n"
              "    pixelSize: %g\n"
              "    pitch: %c",
              family_name.isEmpty() ? "-- first in script --" : family_name.toLatin1().constData(),
-             foundry_name.isEmpty() ? "-- any --" : foundry_name.toLatin1().constData(),
-             script, request.weight, request.style, request.stretch, request.pixelSize, pitch);
+             foundry_name.isEmpty() ? "-- any --" : foundry_name.toLatin1().constData(), script,
+             request.styleName.isEmpty() ? "-- any --" : request.styleName.toLatin1().constData(),
+             request.weight, request.style, request.stretch, request.pixelSize, pitch);
 
     desc->family = nullptr;
     desc->foundry = nullptr;
@@ -1126,6 +1134,10 @@ static int match(int script, const QFontDef &request,
         if (newscore < 10) // xlfd instead of FT... just accept it
             break;
     }
+
+    if (resultingScore != nullptr)
+        *resultingScore = score;
+
     return result;
 }
 
@@ -1239,7 +1251,7 @@ QString QFontDatabase::styleString(const QFontInfo &fontInfo)
 
 /*!
     \fn QFontDatabase::QFontDatabase()
-    \obsolete Call the class methods as static functions instead.
+    \deprecated [6.0] Call the class methods as static functions instead.
 
     Creates a font database object.
 */
@@ -1573,12 +1585,12 @@ bool QFontDatabase::isSmoothlyScalable(const QString &family, const QString &sty
     }
     if (!f) return smoothScalable;
 
-    QtFontStyle::Key styleKey(style);
+    const QtFontStyle::Key styleKey(style);
     for (int j = 0; j < f->count; j++) {
         QtFontFoundry *foundry = f->foundries[j];
         if (foundryName.isEmpty() || foundry->name.compare(foundryName, Qt::CaseInsensitive) == 0) {
             for (int k = 0; k < foundry->count; k++) {
-                QtFontStyle *fontStyle = foundry->styles[k];
+                const QtFontStyle *fontStyle = foundry->styles[k];
                 smoothScalable =
                         fontStyle->smoothScalable
                         && ((style.isEmpty()
@@ -2190,13 +2202,12 @@ bool QFontDatabasePrivate::isApplicationFont(const QString &fileName)
     with removeApplicationFont() or to retrieve the list of family names contained
     in the font.
 
+//! [add-application-font-doc]
     The function returns -1 if the font could not be loaded.
 
     Currently only TrueType fonts, TrueType font collections, and OpenType fonts are
     supported.
-
-    \note Adding application fonts on Unix/X11 platforms without fontconfig is
-    currently not supported.
+//! [add-application-font-doc]
 
     \sa addApplicationFontFromData(), applicationFontFamilies(), removeApplicationFont()
 */
@@ -2224,12 +2235,7 @@ int QFontDatabase::addApplicationFont(const QString &fileName)
     with removeApplicationFont() or to retrieve the list of family names contained
     in the font.
 
-    The function returns -1 if the font could not be loaded.
-
-    Currently only TrueType fonts and TrueType font collections are supported.
-
-    \b{Note:} Adding application fonts on Unix/X11 platforms without fontconfig is
-    currently not supported.
+    \include qfontdatabase.cpp add-application-font-doc
 
     \sa addApplicationFont(), applicationFontFamilies(), removeApplicationFont()
 */
@@ -2350,7 +2356,9 @@ bool QFontDatabase::removeAllApplicationFonts()
 /*!
     \internal
 */
-QFontEngine *QFontDatabasePrivate::findFont(const QFontDef &request, int script)
+QFontEngine *QFontDatabasePrivate::findFont(const QFontDef &req,
+                                            int script,
+                                            bool preferScriptOverFamily)
 {
     QMutexLocker locker(fontDatabaseMutex());
 
@@ -2359,6 +2367,14 @@ QFontEngine *QFontDatabasePrivate::findFont(const QFontDef &request, int script)
 
     QFontEngine *engine;
 
+#ifdef Q_OS_WIN
+    const QFontDef request = static_cast<QWindowsFontDatabaseBase *>(
+                                     QGuiApplicationPrivate::platformIntegration()->fontDatabase())
+                                     ->sanitizeRequest(req);
+#else
+    const QFontDef &request = req;
+#endif
+
 #if defined(QT_BUILD_INTERNAL)
     // For testing purpose only, emulates an exact-matching monospace font
     if (qt_enable_test_font && request.families.first() == QLatin1String("__Qt__Box__Engine__")) {
@@ -2392,11 +2408,20 @@ QFontEngine *QFontDatabasePrivate::findFont(const QFontDef &request, int script)
     parseFontName(requestFamily, foundry_name, family_name);
     QtFontDesc desc;
     QList<int> blackListed;
-    int index = match(multi ? QChar::Script_Common : script, request, family_name, foundry_name, &desc, blackListed);
-    if (index < 0 && QGuiApplicationPrivate::platformIntegration()->fontDatabase()->populateFamilyAliases(family_name)) {
-        // We populated familiy aliases (e.g. localized families), so try again
+    unsigned int score = UINT_MAX;
+    int index = match(multi ? QChar::Script_Common : script, request, family_name, foundry_name, &desc, blackListed, &score);
+    if (score > 0 && QGuiApplicationPrivate::platformIntegration()->fontDatabase()->populateFamilyAliases(family_name)) {
+        // We populated family aliases (e.g. localized families), so try again
         index = match(multi ? QChar::Script_Common : script, request, family_name, foundry_name, &desc, blackListed);
     }
+
+    // If we do not find a match and NoFontMerging is set, use the requested font even if it does
+    // not support the script.
+    //
+    // (we do this at the end to prefer foundries that support the script if they exist)
+    if (index < 0 && !multi && !preferScriptOverFamily)
+        index = match(QChar::Script_Common, request, family_name, foundry_name, &desc, blackListed);
+
     if (index >= 0) {
         QFontDef fontDef = request;
         // Don't pass empty family names to the platform font database, since it will then invoke its own matching
@@ -2498,7 +2523,7 @@ void QFontDatabasePrivate::load(const QFontPrivate *d, int script)
 
     QFontEngine *fe = nullptr;
 
-    Q_TRACE(QFontDatabase_load, req.family, req.pointSize);
+    Q_TRACE(QFontDatabase_load, req.families, req.pointSize);
 
     req.fallBackFamilies = fallBackFamilies;
     if (!req.fallBackFamilies.isEmpty())
