@@ -37,9 +37,10 @@
 **
 ****************************************************************************/
 
+#include <AppKit/AppKit.h>
+
 #include "qcocoamenuloader.h"
 
-#include "messages.h"
 #include "qcocoahelpers.h"
 #include "qcocoansmenu.h"
 #include "qcocoamenubar.h"
@@ -319,12 +320,8 @@
 - (BOOL)validateMenuItem:(NSMenuItem*)menuItem
 {
     if (menuItem.action == @selector(hideOtherApplications:)
-        || menuItem.action == @selector(unhideAllApplications:))
-        return [NSApp validateMenuItem:menuItem];
-
-    if (menuItem.action == @selector(hide:)) {
-        if (QCocoaIntegration::instance()->activePopupWindow())
-            return NO;
+        || menuItem.action == @selector(unhideAllApplications:)
+        || menuItem.action == @selector(hide:)) {
         return [NSApp validateMenuItem:menuItem];
     }
 
