@@ -37,6 +37,8 @@
 **
 ****************************************************************************/
 
+#include <AppKit/AppKit.h>
+
 #include "qnswindowdelegate.h"
 #include "qcocoahelpers.h"
 #include "qcocoawindow.h"
@@ -47,11 +49,19 @@
 #include <qpa/qplatformscreen.h>
 #include <qpa/qwindowsysteminterface.h>
 
-static QRegExp whitespaceRegex = QRegExp(QStringLiteral("\\s*"));
+static inline bool isWhiteSpace(const QString &s)
+{
+    for (int i = 0; i < s.size(); ++i)
+        if (!s.at(i).isSpace())
+            return false;
+    return true;
+}
 
 static QCocoaWindow *toPlatformWindow(NSWindow *window)
 {
-    return qnsview_cast(window.contentView).platformWindow;
+    if ([window conformsToProtocol:@protocol(QNSWindowProtocol)])
+        return static_cast<QCocoaNSWindow *>(window).platformWindow;
+    return nullptr;
 }
 
 @implementation QNSWindowDelegate
@@ -102,40 +112,14 @@ static QCocoaWindow *toPlatformWindow(NSWindow *window)
     return QCocoaScreen::mapToNative(maximizedFrame);
 }
 
-#pragma clang diagnostic push
-// NSDisableScreenUpdates and NSEnableScreenUpdates are deprecated, but the
-// NSAnimationContext API that replaces them doesn't handle the use-case of
-// cross-thread screen update synchronization.
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-- (NSSize)windowWillResize:(NSWindow *)window toSize:(NSSize)frameSize
+- (BOOL)windowShouldZoom:(NSWindow*)window toFrame:(NSRect)newFrame
 {
-    Q_ASSERT(toPlatformWindow(window));
-
-    qCDebug(lcQpaWindow) << window << "will resize to" << QSizeF::fromCGSize(frameSize)
-        << "- disabling screen updates temporarily";
-
-    // There may be separate threads rendering to CA layers in this window,
-    // and if any of them do a swap while the resize is still in progress,
-    // the visual bounds of that layer will be updated before the visual
-    // bounds of the window frame, resulting in flickering while resizing.
-
-    // To prevent this we disable screen updates for the whole process until
-    // the resize is complete, which makes the whole thing visually atomic.
-    NSDisableScreenUpdates();
-
-    return frameSize;
+    QCocoaWindow *platformWindow = toPlatformWindow(window);
+    Q_ASSERT(platformWindow);
+    platformWindow->windowWillZoom();
+    return YES;
 }
 
-- (void)windowDidResize:(NSNotification *)notification
-{
-    NSWindow *window = notification.object;
-    Q_ASSERT(toPlatformWindow(window));
-
-    qCDebug(lcQpaWindow) << window << "was resized - re-enabling screen updates";
-    NSEnableScreenUpdates();
-}
-#pragma clang diagnostic pop
-
 - (BOOL)window:(NSWindow *)window shouldPopUpDocumentPathMenu:(NSMenu *)menu
 {
     Q_UNUSED(menu);
@@ -145,7 +129,7 @@ static QCocoaWindow *toPlatformWindow(NSWindow *window)
 
     // Only pop up document path if the filename is non-empty. We allow whitespace, to
     // allow faking a window icon by setting the file path to a single space character.
-    return !whitespaceRegex.exactMatch(platformWindow->window()->filePath());
+    return !isWhiteSpace(platformWindow->window()->filePath());
 }
 
 - (BOOL)window:(NSWindow *)window shouldDragDocumentWithEvent:(NSEvent *)event from:(NSPoint)dragImageLocation withPasteboard:(NSPasteboard *)pasteboard
@@ -159,6 +143,6 @@ static QCocoaWindow *toPlatformWindow(NSWindow *window)
 
     // Only allow drag if the filename is non-empty. We allow whitespace, to
     // allow faking a window icon by setting the file path to a single space.
-    return !whitespaceRegex.exactMatch(platformWindow->window()->filePath());
+    return !isWhiteSpace(platformWindow->window()->filePath());
 }
 @end
