@@ -785,6 +785,27 @@ static inline QString messageKeyText(const MSG &msg)
     return ch.isNull() ? QString() : QString(ch);
 }
 
+[[nodiscard]] static inline int getTitleBarHeight(const HWND hwnd)
+{
+    const UINT dpi = GetDpiForWindow(hwnd);
+    const int captionHeight = GetSystemMetricsForDpi(SM_CYCAPTION, dpi);
+    if (IsZoomed(hwnd))
+        return captionHeight;
+    // The frame height should also be taken into account if the window
+    // is not maximized.
+    const int frameHeight = GetSystemMetricsForDpi(SM_CYSIZEFRAME, dpi)
+                            + GetSystemMetricsForDpi(SM_CXPADDEDBORDER, dpi);
+    return captionHeight + frameHeight;
+}
+
+[[nodiscard]] static inline bool isSystemMenuOffsetNeeded(const Qt::WindowFlags flags)
+{
+    static constexpr const Qt::WindowFlags titleBarHints =
+        Qt::WindowMinMaxButtonsHint | Qt::WindowCloseButtonHint | Qt::WindowContextHelpButtonHint;
+    return (flags & Qt::WindowSystemMenuHint) && (flags & Qt::WindowTitleHint) && !(flags & titleBarHints)
+           && (flags & (Qt::FramelessWindowHint | Qt::CustomizeWindowHint));
+}
+
 static void showSystemMenu(QWindow* w)
 {
     QWindow *topLevel = QWindowsWindow::topLevelOf(w);
@@ -800,7 +821,6 @@ static void showSystemMenu(QWindow* w)
     bool maximized = IsZoomed(topLevelHwnd);
 
     EnableMenuItem(menu, SC_MAXIMIZE, ! (topLevel->flags() & Qt::WindowMaximizeButtonHint) || maximized?disabled:enabled);
-    EnableMenuItem(menu, SC_RESTORE, maximized?enabled:disabled);
 
     // We should _not_ check with the setFixedSize(x,y) case here, since Windows is not able to check
     // this and our menu here would be out-of-sync with the menu produced by mouse-click on the
@@ -808,6 +828,15 @@ static void showSystemMenu(QWindow* w)
     EnableMenuItem(menu, SC_SIZE, (topLevel->flags() & Qt::MSWindowsFixedSizeDialogHint) || maximized?disabled:enabled);
     EnableMenuItem(menu, SC_MOVE, maximized?disabled:enabled);
     EnableMenuItem(menu, SC_CLOSE, enabled);
+
+    // Highlight the first entry in the menu, this is what native Win32 applications usually do.
+    MENUITEMINFOW restoreItem;
+    SecureZeroMemory(&restoreItem, sizeof(restoreItem));
+    restoreItem.cbSize = sizeof(restoreItem);
+    restoreItem.fMask = MIIM_STATE;
+    restoreItem.fState = MFS_HILITE | (maximized ? MFS_ENABLED : MFS_GRAYED);
+    SetMenuItemInfoW(menu, SC_RESTORE, FALSE, &restoreItem);
+
     // Set bold on close menu item
     MENUITEMINFO closeItem;
     closeItem.cbSize = sizeof(MENUITEMINFO);
@@ -818,9 +847,10 @@ static void showSystemMenu(QWindow* w)
 #undef enabled
 #undef disabled
     const QPoint pos = QHighDpi::toNativePixels(topLevel->geometry().topLeft(), topLevel);
+    const int titleBarOffset = isSystemMenuOffsetNeeded(topLevel->flags()) ? getTitleBarHeight(topLevelHwnd) : 0;
     const int ret = TrackPopupMenuEx(menu,
                                TPM_LEFTALIGN  | TPM_TOPALIGN | TPM_NONOTIFY | TPM_RETURNCMD,
-                               pos.x(), pos.y(),
+                               pos.x(), pos.y() + titleBarOffset,
                                topLevelHwnd,
                                nullptr);
     if (ret)
