@@ -47,7 +47,7 @@
 #include <qpa/qwindowsysteminterface.h>
 #include <QtGui/qguiapplication.h>
 #include <QtGui/qscreen.h>
-#include <QtGui/qtouchdevice.h>
+#include <QtGui/qpointingdevice.h>
 #include <QtGui/qwindow.h>
 #include <QtGui/qcursor.h>
 
@@ -106,7 +106,7 @@ static inline void compressMouseMove(MSG *msg)
                 // Extract the x,y coordinates from the lParam as we do in the WndProc
                 msg->pt.x = GET_X_LPARAM(mouseMsg.lParam);
                 msg->pt.y = GET_Y_LPARAM(mouseMsg.lParam);
-                ClientToScreen(msg->hwnd, &(msg->pt));
+                clientToScreen(msg->hwnd, &(msg->pt));
                 // Remove the mouse move message
                 PeekMessage(&mouseMsg, msg->hwnd, WM_MOUSEMOVE,
                             WM_MOUSEMOVE, PM_REMOVE);
@@ -117,27 +117,6 @@ static inline void compressMouseMove(MSG *msg)
     }
 }
 
-static inline QTouchDevice *createTouchDevice()
-{
-    const int digitizers = GetSystemMetrics(SM_DIGITIZER);
-    if (!(digitizers & (NID_INTEGRATED_TOUCH | NID_EXTERNAL_TOUCH)))
-        return nullptr;
-    const int tabletPc = GetSystemMetrics(SM_TABLETPC);
-    const int maxTouchPoints = GetSystemMetrics(SM_MAXIMUMTOUCHES);
-    qCDebug(lcQpaEvents) << "Digitizers:" << hex << showbase << (digitizers & ~NID_READY)
-        << "Ready:" << (digitizers & NID_READY) << dec << noshowbase
-        << "Tablet PC:" << tabletPc << "Max touch points:" << maxTouchPoints;
-    QTouchDevice *result = new QTouchDevice;
-    result->setType(digitizers & NID_INTEGRATED_TOUCH
-                    ? QTouchDevice::TouchScreen : QTouchDevice::TouchPad);
-    QTouchDevice::Capabilities capabilities = QTouchDevice::Position | QTouchDevice::Area | QTouchDevice::NormalizedPosition;
-    if (result->type() == QTouchDevice::TouchPad)
-        capabilities |= QTouchDevice::MouseEmulation;
-    result->setCapabilities(capabilities);
-    result->setMaximumTouchPoints(maxTouchPoints);
-    return result;
-}
-
 /*!
     \class QWindowsMouseHandler
     \brief Windows mouse handler
@@ -145,28 +124,34 @@ static inline QTouchDevice *createTouchDevice()
     Dispatches mouse and touch events. Separate for code cleanliness.
 
     \internal
-    \ingroup qt-lighthouse-win
 */
 
 QWindowsMouseHandler::QWindowsMouseHandler() = default;
 
-QTouchDevice *QWindowsMouseHandler::ensureTouchDevice()
+const QPointingDevice *QWindowsMouseHandler::primaryMouse()
 {
-    if (!m_touchDevice)
-        m_touchDevice = createTouchDevice();
-    return m_touchDevice;
+    static QPointer<const QPointingDevice> result;
+    if (!result)
+        result = QPointingDevice::primaryPointingDevice();
+    return result;
+}
+
+void QWindowsMouseHandler::clearEvents()
+{
+    m_lastEventType = QEvent::None;
+    m_lastEventButton = Qt::NoButton;
 }
 
 Qt::MouseButtons QWindowsMouseHandler::queryMouseButtons()
 {
-    Qt::MouseButtons result = nullptr;
+    Qt::MouseButtons result;
     const bool mouseSwapped = GetSystemMetrics(SM_SWAPBUTTON);
     if (GetAsyncKeyState(VK_LBUTTON) < 0)
         result |= mouseSwapped ? Qt::RightButton: Qt::LeftButton;
     if (GetAsyncKeyState(VK_RBUTTON) < 0)
         result |= mouseSwapped ? Qt::LeftButton : Qt::RightButton;
     if (GetAsyncKeyState(VK_MBUTTON) < 0)
-        result |= Qt::MidButton;
+        result |= Qt::MiddleButton;
     if (GetAsyncKeyState(VK_XBUTTON1) < 0)
         result |= Qt::XButton1;
     if (GetAsyncKeyState(VK_XBUTTON2) < 0)
@@ -210,11 +195,11 @@ static inline MouseEvent eventFromMsg(const MSG &msg)
     case WM_LBUTTONDBLCLK: // Qt QPA does not handle double clicks, send as press
         return {QEvent::MouseButtonPress, Qt::LeftButton};
     case WM_MBUTTONDOWN:
-        return {QEvent::MouseButtonPress, Qt::MidButton};
+        return {QEvent::MouseButtonPress, Qt::MiddleButton};
     case WM_MBUTTONUP:
-        return {QEvent::MouseButtonRelease, Qt::MidButton};
+        return {QEvent::MouseButtonRelease, Qt::MiddleButton};
     case WM_MBUTTONDBLCLK:
-        return {QEvent::MouseButtonPress, Qt::MidButton};
+        return {QEvent::MouseButtonPress, Qt::MiddleButton};
     case WM_RBUTTONDOWN:
         return {QEvent::MouseButtonPress, Qt::RightButton};
     case WM_RBUTTONUP:
@@ -236,11 +221,11 @@ static inline MouseEvent eventFromMsg(const MSG &msg)
     case WM_NCLBUTTONDBLCLK:
         return {QEvent::NonClientAreaMouseButtonPress, Qt::LeftButton};
     case WM_NCMBUTTONDOWN:
-        return {QEvent::NonClientAreaMouseButtonPress, Qt::MidButton};
+        return {QEvent::NonClientAreaMouseButtonPress, Qt::MiddleButton};
     case WM_NCMBUTTONUP:
-        return {QEvent::NonClientAreaMouseButtonRelease, Qt::MidButton};
+        return {QEvent::NonClientAreaMouseButtonRelease, Qt::MiddleButton};
     case WM_NCMBUTTONDBLCLK:
-        return {QEvent::NonClientAreaMouseButtonPress, Qt::MidButton};
+        return {QEvent::NonClientAreaMouseButtonPress, Qt::MiddleButton};
     case WM_NCRBUTTONDOWN:
         return {QEvent::NonClientAreaMouseButtonPress, Qt::RightButton};
     case WM_NCRBUTTONUP:
@@ -262,15 +247,26 @@ bool QWindowsMouseHandler::translateMouseEvent(QWindow *window, HWND hwnd,
     if (et == QtWindows::MouseWheelEvent)
         return translateMouseWheelEvent(window, hwnd, msg, result);
 
-    const QPoint winEventPosition(GET_X_LPARAM(msg.lParam), GET_Y_LPARAM(msg.lParam));
+    QPoint winEventPosition(GET_X_LPARAM(msg.lParam), GET_Y_LPARAM(msg.lParam));
+    if ((et & QtWindows::NonClientEventFlag) == 0 && QWindowsBaseWindow::isRtlLayout(hwnd))  {
+        RECT clientArea;
+        GetClientRect(hwnd, &clientArea);
+        winEventPosition.setX(clientArea.right - winEventPosition.x());
+    }
+
     QPoint clientPosition;
     QPoint globalPosition;
     if (et & QtWindows::NonClientEventFlag) {
         globalPosition = winEventPosition;
         clientPosition = QWindowsGeometryHint::mapFromGlobal(hwnd, globalPosition);
     } else {
-        clientPosition = winEventPosition;
         globalPosition = QWindowsGeometryHint::mapToGlobal(hwnd, winEventPosition);
+        auto targetHwnd = hwnd;
+        if (auto *pw = window->handle())
+            targetHwnd = HWND(pw->winId());
+        clientPosition = targetHwnd == hwnd
+            ? winEventPosition
+            : QWindowsGeometryHint::mapFromGlobal(targetHwnd, globalPosition);
     }
 
     // Windows sends a mouse move with no buttons pressed to signal "Enter"
@@ -287,7 +283,7 @@ bool QWindowsMouseHandler::translateMouseEvent(QWindow *window, HWND hwnd,
 
     Qt::MouseEventSource source = Qt::MouseEventNotSynthesized;
 
-    const MouseEvent mouseEvent = eventFromMsg(msg);
+    const QPointingDevice *device = primaryMouse();
 
     // Check for events synthesized from touch. Lower byte is touch index, 0 means pen.
     static const bool passSynthesizedMouseEvents =
@@ -296,22 +292,51 @@ bool QWindowsMouseHandler::translateMouseEvent(QWindow *window, HWND hwnd,
     // However, when tablet support is active, extraInfo is a packet serial number. This is not a problem
     // since we do not want to ignore mouse events coming from a tablet.
     // See https://msdn.microsoft.com/en-us/library/windows/desktop/ms703320.aspx
-    const quint64 extraInfo = quint64(GetMessageExtraInfo());
+    const auto extraInfo = quint64(GetMessageExtraInfo());
     if ((extraInfo & signatureMask) == miWpSignature) {
         if (extraInfo & 0x80) { // Bit 7 indicates touch event, else tablet pen.
             source = Qt::MouseEventSynthesizedBySystem;
+            if (!m_touchDevice.isNull())
+                device = m_touchDevice.data();
             if (!passSynthesizedMouseEvents)
                 return false;
         }
     }
 
+    const Qt::KeyboardModifiers keyModifiers = QWindowsKeyMapper::queryKeyboardModifiers();
+    const MouseEvent mouseEvent = eventFromMsg(msg);
+    Qt::MouseButtons buttons;
+
+    if (mouseEvent.type >= QEvent::NonClientAreaMouseMove && mouseEvent.type <= QEvent::NonClientAreaMouseButtonDblClick)
+        buttons = queryMouseButtons();
+    else
+        buttons = keyStateToMouseButtons(msg.wParam);
+
+    // When the left/right mouse buttons are pressed over the window title bar
+    // WM_NCLBUTTONDOWN/WM_NCRBUTTONDOWN messages are received. But no UP
+    // messages are received on release, only WM_NCMOUSEMOVE/WM_MOUSEMOVE.
+    // We detect it and generate the missing release events here. (QTBUG-75678)
+    // The last event vars are cleared on QWindowsContext::handleExitSizeMove()
+    // to avoid generating duplicated release events.
+    if (m_lastEventType == QEvent::NonClientAreaMouseButtonPress
+            && (mouseEvent.type == QEvent::NonClientAreaMouseMove || mouseEvent.type == QEvent::MouseMove)
+            && (m_lastEventButton & buttons) == 0) {
+            if (mouseEvent.type == QEvent::NonClientAreaMouseMove) {
+                QWindowSystemInterface::handleFrameStrutMouseEvent(window, device, clientPosition, globalPosition, buttons, m_lastEventButton,
+                                                                   QEvent::NonClientAreaMouseButtonRelease, keyModifiers, source);
+            } else {
+                QWindowSystemInterface::handleMouseEvent(window, device, clientPosition, globalPosition, buttons, m_lastEventButton,
+                                                         QEvent::MouseButtonRelease, keyModifiers, source);
+            }
+    }
+    m_lastEventType = mouseEvent.type;
+    m_lastEventButton = mouseEvent.button;
+
     if (mouseEvent.type >= QEvent::NonClientAreaMouseMove && mouseEvent.type <= QEvent::NonClientAreaMouseButtonDblClick) {
-        const Qt::MouseButtons buttons = QWindowsMouseHandler::queryMouseButtons();
-        QWindowSystemInterface::handleFrameStrutMouseEvent(window, clientPosition,
+        QWindowSystemInterface::handleFrameStrutMouseEvent(window, device, clientPosition,
                                                            globalPosition, buttons,
                                                            mouseEvent.button, mouseEvent.type,
-                                                           QWindowsKeyMapper::queryKeyboardModifiers(),
-                                                           source);
+                                                           keyModifiers, source);
         return false; // Allow further event processing (dragging of windows).
     }
 
@@ -333,10 +358,9 @@ bool QWindowsMouseHandler::translateMouseEvent(QWindow *window, HWND hwnd,
         return true;
     }
 
-    QWindowsWindow *platformWindow = static_cast<QWindowsWindow *>(window->handle());
-    const Qt::MouseButtons buttons = keyStateToMouseButtons(int(msg.wParam));
+    auto *platformWindow = static_cast<QWindowsWindow *>(window->handle());
 
-    // If the window was recently resized via mouse doubleclick on the frame or title bar,
+    // If the window was recently resized via mouse double-click on the frame or title bar,
     // we don't get WM_LBUTTONDOWN or WM_LBUTTONDBLCLK for the second click,
     // but we will get at least one WM_MOUSEMOVE with left button down and the WM_LBUTTONUP,
     // which will result undesired mouse press and release events.
@@ -459,10 +483,9 @@ bool QWindowsMouseHandler::translateMouseEvent(QWindow *window, HWND hwnd,
     }
 
     if (!discardEvent && mouseEvent.type != QEvent::None) {
-        QWindowSystemInterface::handleMouseEvent(window, winEventPosition, globalPosition, buttons,
+        QWindowSystemInterface::handleMouseEvent(window, device, clientPosition, globalPosition, buttons,
                                                  mouseEvent.button, mouseEvent.type,
-                                                 QWindowsKeyMapper::queryKeyboardModifiers(),
-                                                 source);
+                                                 keyModifiers, source);
     }
     m_previousCaptureWindow = hasCapture ? window : nullptr;
     // QTBUG-48117, force synchronous handling for the extra buttons so that WM_APPCOMMAND
@@ -574,8 +597,8 @@ bool QWindowsMouseHandler::translateTouchEvent(QWindow *window, HWND,
                                                QtWindows::WindowsEventType,
                                                MSG msg, LRESULT *)
 {
-    typedef QWindowSystemInterface::TouchPoint QTouchPoint;
-    typedef QList<QWindowSystemInterface::TouchPoint> QTouchPointList;
+    using QTouchPoint = QWindowSystemInterface::TouchPoint;
+    using QTouchPointList = QList<QWindowSystemInterface::TouchPoint>;
 
     if (!QWindowsContext::instance()->initTouch()) {
         qWarning("Unable to initialize touch handling.");
@@ -595,7 +618,7 @@ bool QWindowsMouseHandler::translateTouchEvent(QWindow *window, HWND,
 
     QTouchPointList touchPoints;
     touchPoints.reserve(winTouchPointCount);
-    Qt::TouchPointStates allStates = nullptr;
+    QEventPoint::States allStates;
 
     GetTouchInputInfo(reinterpret_cast<HTOUCHINPUT>(msg.lParam),
                       UINT(msg.wParam), winTouchInputs.data(), sizeof(TOUCHINPUT));
@@ -622,15 +645,15 @@ bool QWindowsMouseHandler::translateTouchEvent(QWindow *window, HWND,
         touchPoint.normalPosition = normalPosition;
 
         if (winTouchInput.dwFlags & TOUCHEVENTF_DOWN) {
-            touchPoint.state = Qt::TouchPointPressed;
+            touchPoint.state = QEventPoint::State::Pressed;
             m_lastTouchPositions.insert(id, touchPoint.normalPosition);
         } else if (winTouchInput.dwFlags & TOUCHEVENTF_UP) {
-            touchPoint.state = Qt::TouchPointReleased;
+            touchPoint.state = QEventPoint::State::Released;
             m_lastTouchPositions.remove(id);
         } else {
             touchPoint.state = (stationaryTouchPoint
-                     ? Qt::TouchPointStationary
-                     : Qt::TouchPointMoved);
+                     ? QEventPoint::State::Stationary
+                     : QEventPoint::State::Updated);
             m_lastTouchPositions.insert(id, touchPoint.normalPosition);
         }
 
@@ -642,11 +665,11 @@ bool QWindowsMouseHandler::translateTouchEvent(QWindow *window, HWND,
     CloseTouchInputHandle(reinterpret_cast<HTOUCHINPUT>(msg.lParam));
 
     // all touch points released, forget the ids we've seen, they may not be reused
-    if (allStates == Qt::TouchPointReleased)
+    if (allStates == QEventPoint::State::Released)
         m_touchInputIDToTouchPointID.clear();
 
     QWindowSystemInterface::handleTouchEvent(window,
-                                             m_touchDevice,
+                                             m_touchDevice.data(),
                                              touchPoints,
                                              QWindowsKeyMapper::queryKeyboardModifiers());
     return true;
@@ -656,9 +679,9 @@ bool QWindowsMouseHandler::translateGestureEvent(QWindow *window, HWND hwnd,
                                                  QtWindows::WindowsEventType,
                                                  MSG msg, LRESULT *)
 {
-    Q_UNUSED(window)
-    Q_UNUSED(hwnd)
-    Q_UNUSED(msg)
+    Q_UNUSED(window);
+    Q_UNUSED(hwnd);
+    Q_UNUSED(msg);
     return false;
 }
 
