@@ -75,7 +75,7 @@ QT_BEGIN_NAMESPACE
 using namespace QWindowsUiAutomation;
 
 
-// Returns a cached instance of the provider for a specific acessible interface.
+// Returns a cached instance of the provider for a specific accessible interface.
 QWindowsUiaMainProvider *QWindowsUiaMainProvider::providerForAccessible(QAccessibleInterface *accessible)
 {
     if (!accessible)
@@ -107,6 +107,17 @@ QWindowsUiaMainProvider::~QWindowsUiaMainProvider()
 void QWindowsUiaMainProvider::notifyFocusChange(QAccessibleEvent *event)
 {
     if (QAccessibleInterface *accessible = event->accessibleInterface()) {
+        // If this is a table/tree/list, raise event for the focused cell/item instead.
+        if (accessible->tableInterface()) {
+            int count = accessible->childCount();
+            for (int i = 0; i < count; ++i) {
+                QAccessibleInterface *item = accessible->child(i);
+                if (item && item->isValid() && item->state().focused) {
+                    accessible = item;
+                    break;
+                }
+            }
+        }
         if (QWindowsUiaMainProvider *provider = providerForAccessible(accessible)) {
             QWindowsUiaWrapper::instance()->raiseAutomationEvent(provider, UIA_AutomationFocusChangedEventId);
         }
@@ -429,7 +440,7 @@ HRESULT QWindowsUiaMainProvider::GetPropertyValue(PROPERTYID idProp, VARIANT *pR
             // Control type converted from role.
             auto controlType = roleToControlTypeId(accessible->role());
 
-            // The native OSK should be disbled if the Qt OSK is in use,
+            // The native OSK should be disabled if the Qt OSK is in use,
             // or if disabled via application attribute.
             static bool imModuleEmpty = qEnvironmentVariableIsEmpty("QT_IM_MODULE");
             bool nativeVKDisabled = QCoreApplication::testAttribute(Qt::AA_DisableNativeVirtualKeyboard);
@@ -513,7 +524,7 @@ QString QWindowsUiaMainProvider::automationIdForAccessible(const QAccessibleInte
         while (obj) {
             QString name = obj->objectName();
             if (name.isEmpty())
-                return QString();
+                return result;
             if (!result.isEmpty())
                 result.prepend(u'.');
             result.prepend(name);
