@@ -71,7 +71,7 @@ public:
             break;
 
         default:
-            qWarning("QXcbClipboardMime: Internal error: Unsupported clipboard mode");
+            qCWarning(lcQpaClipboard, "QXcbClipboardMime: Internal error: Unsupported clipboard mode");
             break;
         }
     }
@@ -83,7 +83,7 @@ public:
 
     bool isEmpty() const
     {
-        return m_clipboard->getSelectionOwner(modeAtom) == XCB_NONE;
+        return m_clipboard->connection()->selectionOwner(modeAtom) == XCB_NONE;
     }
 
 protected:
@@ -231,14 +231,15 @@ QXcbClipboard::QXcbClipboard(QXcbConnection *c)
     m_clientClipboard[QClipboard::Selection] = nullptr;
     m_timestamp[QClipboard::Clipboard] = XCB_CURRENT_TIME;
     m_timestamp[QClipboard::Selection] = XCB_CURRENT_TIME;
-    m_owner = connection()->getQtSelectionOwner();
 
     if (connection()->hasXFixes()) {
         const uint32_t mask = XCB_XFIXES_SELECTION_EVENT_MASK_SET_SELECTION_OWNER |
                 XCB_XFIXES_SELECTION_EVENT_MASK_SELECTION_WINDOW_DESTROY |
                 XCB_XFIXES_SELECTION_EVENT_MASK_SELECTION_CLIENT_CLOSE;
-        xcb_xfixes_select_selection_input_checked(xcb_connection(), m_owner, XCB_ATOM_PRIMARY, mask);
-        xcb_xfixes_select_selection_input_checked(xcb_connection(), m_owner, atom(QXcbAtom::CLIPBOARD), mask);
+        xcb_xfixes_select_selection_input_checked(xcb_connection(), connection()->qtSelectionOwner(),
+                                                  XCB_ATOM_PRIMARY, mask);
+        xcb_xfixes_select_selection_input_checked(xcb_connection(), connection()->qtSelectionOwner(),
+                                                  atom(QXcbAtom::CLIPBOARD), mask);
     }
 
     // xcb_change_property_request_t and xcb_get_property_request_t are the same size
@@ -253,19 +254,21 @@ QXcbClipboard::~QXcbClipboard()
             m_timestamp[QClipboard::Selection] != XCB_CURRENT_TIME) {
 
         // First we check if there is a clipboard manager.
-        auto reply = Q_XCB_REPLY(xcb_get_selection_owner, xcb_connection(), atom(QXcbAtom::CLIPBOARD_MANAGER));
-        if (reply && reply->owner != XCB_NONE) {
+        if (connection()->selectionOwner(atom(QXcbAtom::CLIPBOARD_MANAGER)) != XCB_NONE) {
             // we delete the property so the manager saves all TARGETS.
-            xcb_delete_property(xcb_connection(), m_owner, atom(QXcbAtom::_QT_SELECTION));
-            xcb_convert_selection(xcb_connection(), m_owner, atom(QXcbAtom::CLIPBOARD_MANAGER), atom(QXcbAtom::SAVE_TARGETS),
+            xcb_delete_property(xcb_connection(), connection()->qtSelectionOwner(),
+                                atom(QXcbAtom::_QT_SELECTION));
+            xcb_convert_selection(xcb_connection(), connection()->qtSelectionOwner(),
+                                  atom(QXcbAtom::CLIPBOARD_MANAGER), atom(QXcbAtom::SAVE_TARGETS),
                                   atom(QXcbAtom::_QT_SELECTION), connection()->time());
             connection()->sync();
 
             // waiting until the clipboard manager fetches the content.
-            if (auto event = waitForClipboardEvent(m_owner, XCB_SELECTION_NOTIFY, true)) {
+            if (auto event = waitForClipboardEvent(connection()->qtSelectionOwner(),
+                                                   XCB_SELECTION_NOTIFY, true)) {
                 free(event);
             } else {
-                qWarning("QXcbClipboard: Unable to receive an event from the "
+                qCWarning(lcQpaClipboard, "QXcbClipboard: Unable to receive an event from the "
                          "clipboard manager in a reasonable time");
             }
         }
@@ -289,11 +292,6 @@ bool QXcbClipboard::handlePropertyNotify(const xcb_generic_event_t *event)
     return (*it)->updateIncrementalProperty(propertyNotify);
 }
 
-xcb_window_t QXcbClipboard::getSelectionOwner(xcb_atom_t atom) const
-{
-    return connection()->getSelectionOwner(atom);
-}
-
 xcb_atom_t QXcbClipboard::atomForMode(QClipboard::Mode mode) const
 {
     if (mode == QClipboard::Clipboard)
@@ -319,8 +317,8 @@ QMimeData * QXcbClipboard::mimeData(QClipboard::Mode mode)
     if (mode > QClipboard::Selection)
         return nullptr;
 
-    xcb_window_t clipboardOwner = getSelectionOwner(atomForMode(mode));
-    if (clipboardOwner == owner()) {
+    xcb_window_t clipboardOwner = connection()->selectionOwner(atomForMode(mode));
+    if (clipboardOwner == connection()->qtSelectionOwner()) {
         return m_clientClipboard[mode];
     } else {
         if (!m_xClipboard[mode])
@@ -362,7 +360,7 @@ void QXcbClipboard::setMimeData(QMimeData *data, QClipboard::Mode mode)
         connection()->setTime(connection()->getTimestamp());
 
     if (data) {
-        newOwner = owner();
+        newOwner = connection()->qtSelectionOwner();
 
         m_clientClipboard[mode] = data;
         m_timestamp[mode] = connection()->time();
@@ -370,8 +368,8 @@ void QXcbClipboard::setMimeData(QMimeData *data, QClipboard::Mode mode)
 
     xcb_set_selection_owner(xcb_connection(), newOwner, modeAtom, connection()->time());
 
-    if (getSelectionOwner(modeAtom) != newOwner) {
-        qWarning("QXcbClipboard::setMimeData: Cannot set X11 selection owner");
+    if (connection()->selectionOwner(modeAtom) != newOwner) {
+        qCWarning(lcQpaClipboard, "QXcbClipboard::setMimeData: Cannot set X11 selection owner");
     }
 
     emitChanged(mode);
@@ -386,10 +384,11 @@ bool QXcbClipboard::supportsMode(QClipboard::Mode mode) const
 
 bool QXcbClipboard::ownsMode(QClipboard::Mode mode) const
 {
-    if (m_owner == XCB_NONE || mode > QClipboard::Selection)
+    if (connection()->qtSelectionOwner() == XCB_NONE || mode > QClipboard::Selection)
         return false;
 
-    Q_ASSERT(m_timestamp[mode] == XCB_CURRENT_TIME || getSelectionOwner(atomForMode(mode)) == m_owner);
+    Q_ASSERT(m_timestamp[mode] == XCB_CURRENT_TIME
+             || connection()->selectionOwner(atomForMode(mode)) == connection()->qtSelectionOwner());
 
     return m_timestamp[mode] != XCB_CURRENT_TIME;
 }
@@ -438,11 +437,6 @@ void QXcbClipboard::setRequestor(xcb_window_t window)
     m_requestor = window;
 }
 
-xcb_window_t QXcbClipboard::owner() const
-{
-    return m_owner;
-}
-
 xcb_atom_t QXcbClipboard::sendTargetsSelection(QMimeData *d, xcb_window_t window, xcb_atom_t property)
 {
     QList<xcb_atom_t> types;
@@ -521,7 +515,7 @@ void QXcbClipboard::handleSelectionClearRequest(xcb_selection_clear_event_t *eve
 //          XGetSelectionOwner(dpy, XA_PRIMARY),
 //          xevent->xselectionclear.time, d->timestamp);
 
-    xcb_window_t newOwner = getSelectionOwner(event->selection);
+    xcb_window_t newOwner = connection()->selectionOwner(event->selection);
 
     /* If selection ownership was given up voluntarily from QClipboard::clear(), then we do nothing here
     since its already handled in setMimeData. Otherwise, the event must have come from another client
@@ -538,7 +532,7 @@ void QXcbClipboard::handleSelectionClearRequest(xcb_selection_clear_event_t *eve
 void QXcbClipboard::handleSelectionRequest(xcb_selection_request_event_t *req)
 {
     if (requestor() && req->requestor == requestor()) {
-        qWarning("QXcbClipboard: Selection request should be caught before");
+        qCWarning(lcQpaClipboard, "QXcbClipboard: Selection request should be caught before");
         return;
     }
 
@@ -553,7 +547,8 @@ void QXcbClipboard::handleSelectionRequest(xcb_selection_request_event_t *req)
     QMimeData *d;
     QClipboard::Mode mode = modeForAtom(req->selection);
     if (mode > QClipboard::Selection) {
-        qWarning() << "QXcbClipboard: Unknown selection" << connection()->atomName(req->selection);
+        qCWarning(lcQpaClipboard, "QXcbClipboard: Unknown selection %s",
+                  connection()->atomName(req->selection).constData());
         xcb_send_event(xcb_connection(), false, req->requestor, XCB_EVENT_MASK_NO_EVENT, (const char *)&event);
         return;
     }
@@ -561,14 +556,14 @@ void QXcbClipboard::handleSelectionRequest(xcb_selection_request_event_t *req)
     d = m_clientClipboard[mode];
 
     if (!d) {
-        qWarning("QXcbClipboard: Cannot transfer data, no data available");
+        qCWarning(lcQpaClipboard, "QXcbClipboard: Cannot transfer data, no data available");
         xcb_send_event(xcb_connection(), false, req->requestor, XCB_EVENT_MASK_NO_EVENT, (const char *)&event);
         return;
     }
 
     if (m_timestamp[mode] == XCB_CURRENT_TIME // we don't own the selection anymore
             || (req->time != XCB_CURRENT_TIME && req->time < m_timestamp[mode])) {
-        qWarning("QXcbClipboard: SelectionRequest too old");
+        qCDebug(lcQpaClipboard, "QXcbClipboard: SelectionRequest too old");
         xcb_send_event(xcb_connection(), false, req->requestor, XCB_EVENT_MASK_NO_EVENT, (const char *)&event);
         return;
     }
@@ -623,7 +618,7 @@ void QXcbClipboard::handleSelectionRequest(xcb_selection_request_event_t *req)
                                     property, XCB_ATOM_INTEGER, 32, 1, &m_timestamp[mode]);
                 ret = property;
             } else {
-                qWarning("QXcbClipboard: Invalid data timestamp");
+                qCWarning(lcQpaClipboard, "QXcbClipboard: Invalid data timestamp");
             }
         } else if (target == targetsAtom) {
             ret = sendTargetsSelection(d, req->requestor, property);
@@ -667,7 +662,7 @@ void QXcbClipboard::handleXFixesSelectionRequest(xcb_xfixes_selection_notify_eve
     // Note1: Here we care only about the xfixes events that come from other processes.
     // Note2: If the QClipboard::clear() is issued, event->owner is XCB_NONE,
     // so we check selection_timestamp to not handle our own QClipboard::clear().
-    if (event->owner != owner() && event->selection_timestamp > m_timestamp[mode]) {
+    if (event->owner != connection()->qtSelectionOwner() && event->selection_timestamp > m_timestamp[mode]) {
         if (!m_xClipboard[mode]) {
             m_xClipboard[mode].reset(new QXcbClipboardMime(mode, this));
         } else {
@@ -728,7 +723,7 @@ bool QXcbClipboard::clipboardReadProperty(xcb_window_t win, xcb_atom_t property,
             // recover -- this shouldn't normally happen, but it doesn't
             // hurt to be defensive
             if ((int)(buffer_offset + length) > buffer->size()) {
-                qWarning("QXcbClipboard: buffer overflow");
+                qCWarning(lcQpaClipboard, "QXcbClipboard: buffer overflow");
                 length = buffer->size() - buffer_offset;
 
                 // escape loop
@@ -788,8 +783,7 @@ xcb_generic_event_t *QXcbClipboard::waitForClipboardEvent(xcb_window_t window, i
         const QXcbEventNode *flushedTailNode = queue->flushedTail();
 
         if (checkManager) {
-            auto reply = Q_XCB_REPLY(xcb_get_selection_owner, xcb_connection(), atom(QXcbAtom::CLIPBOARD_MANAGER));
-            if (!reply || reply->owner == XCB_NONE)
+            if (connection()->selectionOwner(atom(QXcbAtom::CLIPBOARD_MANAGER)) == XCB_NONE)
                 return nullptr;
         }
 
@@ -835,6 +829,8 @@ QByteArray QXcbClipboard::clipboardReadIncrementalProperty(xcb_window_t win, xcb
         alloc_error = buf.size() != nbytes+1;
     }
 
+    QElapsedTimer timer;
+    timer.start();
     for (;;) {
         connection()->flush();
         xcb_generic_event_t *ge = waitForClipboardEvent(win, XCB_PROPERTY_NOTIFY);
@@ -870,9 +866,11 @@ QByteArray QXcbClipboard::clipboardReadIncrementalProperty(xcb_window_t win, xcb
                 tmp_buf.resize(0);
                 offset += length;
             }
-        } else {
-            break;
         }
+
+        const auto elapsed = timer.elapsed();
+        if (elapsed > clipboard_timeout)
+            break;
     }
 
     // timed out ... create a new requestor window, otherwise the requestor
