@@ -62,7 +62,7 @@ QXcbVirtualDesktop::QXcbVirtualDesktop(QXcbConnection *connection, xcb_screen_t
 {
     const QByteArray cmAtomName =  "_NET_WM_CM_S" + QByteArray::number(m_number);
     m_net_wm_cm_atom = connection->internAtom(cmAtomName.constData());
-    m_compositingActive = connection->getSelectionOwner(m_net_wm_cm_atom);
+    m_compositingActive = connection->selectionOwner(m_net_wm_cm_atom);
 
     m_workArea = getWorkArea();
 
@@ -112,11 +112,24 @@ QXcbVirtualDesktop::QXcbVirtualDesktop(QXcbConnection *connection, xcb_screen_t
 
         xcb_depth_next(&depth_iterator);
     }
+
+    auto dpiChangedCallback = [](QXcbVirtualDesktop *desktop, const QByteArray &, const QVariant &property, void *) {
+        if (!desktop->setDpiFromXSettings(property))
+            return;
+        const auto dpi = desktop->forcedDpi();
+        for (QXcbScreen *screen : desktop->connection()->screens())
+            QWindowSystemInterface::handleScreenLogicalDotsPerInchChange(screen->QPlatformScreen::screen(), dpi, dpi);
+    };
+    setDpiFromXSettings(xSettings()->setting("Xft/DPI"));
+    xSettings()->registerCallbackForProperty("Xft/DPI", dpiChangedCallback, nullptr);
 }
 
 QXcbVirtualDesktop::~QXcbVirtualDesktop()
 {
     delete m_xSettings;
+
+    for (auto cmap : qAsConst(m_visualColormaps))
+        xcb_free_colormap(xcb_connection(), cmap);
 }
 
 QDpi QXcbVirtualDesktop::dpi() const
@@ -164,7 +177,7 @@ bool QXcbVirtualDesktop::compositingActive() const
     if (connection()->hasXFixes())
         return m_compositingActive;
     else
-        return connection()->getSelectionOwner(m_net_wm_cm_atom);
+        return connection()->selectionOwner(m_net_wm_cm_atom);
 }
 
 void QXcbVirtualDesktop::handleXFixesSelectionNotify(xcb_xfixes_selection_notify_event_t *notify_event)
@@ -179,7 +192,7 @@ void QXcbVirtualDesktop::subscribeToXFixesSelectionNotify()
         const uint32_t mask = XCB_XFIXES_SELECTION_EVENT_MASK_SET_SELECTION_OWNER |
                               XCB_XFIXES_SELECTION_EVENT_MASK_SELECTION_WINDOW_DESTROY |
                               XCB_XFIXES_SELECTION_EVENT_MASK_SELECTION_CLIENT_CLOSE;
-        xcb_xfixes_select_selection_input_checked(xcb_connection(), connection()->getQtSelectionOwner(), m_net_wm_cm_atom, mask);
+        xcb_xfixes_select_selection_input_checked(xcb_connection(), connection()->qtSelectionOwner(), m_net_wm_cm_atom, mask);
     }
 }
 
@@ -245,7 +258,7 @@ void QXcbVirtualDesktop::handleScreenChange(xcb_randr_screen_change_notify_event
     case XCB_RANDR_ROTATION_REFLECT_Y: break;
     }
 
-    for (QPlatformScreen *platformScreen: qAsConst(m_screens)) {
+    for (QPlatformScreen *platformScreen : qAsConst(m_screens)) {
         QDpi ldpi = platformScreen->logicalDpi();
         QWindowSystemInterface::handleScreenLogicalDotsPerInchChange(platformScreen->screen(), ldpi.first, ldpi.second);
     }
@@ -258,7 +271,7 @@ void QXcbVirtualDesktop::handleScreenChange(xcb_randr_screen_change_notify_event
     _NET_WORKAREA means with multiple attached monitors. This gets worse when monitors have
     different dimensions and/or screens are not virtually aligned. In Qt we want the available
     geometry per monitor (QScreen), not desktop (represented by _NET_WORKAREA). WM specification
-    does not have an atom for this. Thus, QScreen is limted by the lack of support from the
+    does not have an atom for this. Thus, QScreen is limited by the lack of support from the
     underlying system.
 
     One option could be that Qt does WM's job of calculating this by subtracting geometries of
@@ -284,7 +297,7 @@ QRect QXcbVirtualDesktop::getWorkArea() const
         uint32_t *geom = (uint32_t*)xcb_get_property_value(workArea.get());
         r = QRect(geom[0], geom[1], geom[2], geom[3]);
     } else {
-        r = QRect(QPoint(), size());
+        r.setWidth(-1);
     }
     return r;
 }
@@ -299,6 +312,11 @@ void QXcbVirtualDesktop::updateWorkArea()
     }
 }
 
+QRect QXcbVirtualDesktop::availableGeometry(const QRect &screenGeometry) const
+{
+    return m_workArea.width() >= 0 ? screenGeometry & m_workArea : screenGeometry;
+}
+
 static inline QSizeF sizeInMillimeters(const QSize &size, const QDpi &dpi)
 {
     return QSizeF(Q_MM_PER_INCH * size.width() / dpi.first,
@@ -318,12 +336,22 @@ bool QXcbVirtualDesktop::xResource(const QByteArray &identifier,
 
 static bool parseXftInt(const QByteArray& stringValue, int *value)
 {
-    Q_ASSERT(value != 0);
+    Q_ASSERT(value);
     bool ok;
     *value = stringValue.toInt(&ok);
     return ok;
 }
 
+static bool parseXftDpi(const QByteArray& stringValue, int *value)
+{
+    Q_ASSERT(value);
+    bool ok = parseXftInt(stringValue, value);
+    // Support GNOME 3 bug that wrote DPI with fraction:
+    if (!ok)
+        *value = qRound(stringValue.toDouble(&ok));
+    return ok;
+}
+
 static QFontEngine::HintStyle parseXftHintStyle(const QByteArray& stringValue)
 {
     if (stringValue == "hintfull")
@@ -380,7 +408,7 @@ void QXcbVirtualDesktop::readXResources()
         int value;
         QByteArray stringValue;
         if (xResource(r, "Xft.dpi:\t", stringValue)) {
-            if (parseXftInt(stringValue, &value))
+            if (parseXftDpi(stringValue, &value))
                 m_forcedDpi = value;
         } else if (xResource(r, "Xft.hintstyle:\t", stringValue)) {
             m_hintStyle = parseXftHintStyle(stringValue);
@@ -393,6 +421,19 @@ void QXcbVirtualDesktop::readXResources()
     }
 }
 
+bool QXcbVirtualDesktop::setDpiFromXSettings(const QVariant &property)
+{
+    bool ok;
+    int dpiTimes1k = property.toInt(&ok);
+    if (!ok)
+        return false;
+    int dpi = dpiTimes1k / 1024;
+    if (m_forcedDpi == dpi)
+        return false;
+    m_forcedDpi = dpi;
+    return true;
+}
+
 QSurfaceFormat QXcbVirtualDesktop::surfaceFormatFor(const QSurfaceFormat &format) const
 {
     const xcb_visualid_t xcb_visualid = connection()->hasDefaultVisualId() ? connection()->defaultVisualId()
@@ -457,7 +498,7 @@ const xcb_visualtype_t *QXcbVirtualDesktop::visualForId(xcb_visualid_t visualid)
 {
     QMap<xcb_visualid_t, xcb_visualtype_t>::const_iterator it = m_visuals.find(visualid);
     if (it == m_visuals.constEnd())
-        return 0;
+        return nullptr;
     return &*it;
 }
 
@@ -469,17 +510,33 @@ quint8 QXcbVirtualDesktop::depthOfVisual(xcb_visualid_t visualid) const
     return *it;
 }
 
+xcb_colormap_t QXcbVirtualDesktop::colormapForVisual(xcb_visualid_t visualid) const
+{
+    auto it = m_visualColormaps.constFind(visualid);
+    if (it != m_visualColormaps.constEnd())
+        return *it;
+
+    auto cmap = xcb_generate_id(xcb_connection());
+    xcb_create_colormap(xcb_connection(),
+                        XCB_COLORMAP_ALLOC_NONE,
+                        cmap,
+                        screen()->root,
+                        visualid);
+    m_visualColormaps.insert(visualid, cmap);
+    return cmap;
+}
+
 QXcbScreen::QXcbScreen(QXcbConnection *connection, QXcbVirtualDesktop *virtualDesktop,
-                       xcb_randr_output_t outputId, xcb_randr_get_output_info_reply_t *output,
-                       const xcb_xinerama_screen_info_t *xineramaScreenInfo, int xineramaScreenIdx)
+                       xcb_randr_output_t outputId, xcb_randr_get_output_info_reply_t *output)
     : QXcbObject(connection)
     , m_virtualDesktop(virtualDesktop)
+    , m_monitor(nullptr)
     , m_output(outputId)
     , m_crtc(output ? output->crtc : XCB_NONE)
     , m_outputName(getOutputName(output))
     , m_outputSizeMillimeters(output ? QSize(output->mm_width, output->mm_height) : QSize())
 {
-    if (connection->hasXRandr()) {
+    if (connection->isAtLeastXRandR12()) {
         xcb_randr_select_input(xcb_connection(), screen()->root, true);
         auto crtc = Q_XCB_REPLY_UNCHECKED(xcb_randr_get_crtc_info, xcb_connection(),
                                           m_crtc, output ? output->timestamp : 0);
@@ -487,27 +544,31 @@ QXcbScreen::QXcbScreen(QXcbConnection *connection, QXcbVirtualDesktop *virtualDe
             updateGeometry(QRect(crtc->x, crtc->y, crtc->width, crtc->height), crtc->rotation);
             updateRefreshRate(crtc->mode);
         }
-    } else if (xineramaScreenInfo) {
-        m_geometry = QRect(xineramaScreenInfo->x_org, xineramaScreenInfo->y_org,
-                           xineramaScreenInfo->width, xineramaScreenInfo->height);
-        m_availableGeometry = m_geometry & m_virtualDesktop->workArea();
-        m_sizeMillimeters = sizeInMillimeters(m_geometry.size(), m_virtualDesktop->dpi());
-        if (xineramaScreenIdx > -1)
-            m_outputName += QLatin1Char('-') + QString::number(xineramaScreenIdx);
     }
 
     if (m_geometry.isEmpty())
         m_geometry = QRect(QPoint(), virtualDesktop->size());
 
     if (m_availableGeometry.isEmpty())
-        m_availableGeometry = m_geometry & m_virtualDesktop->workArea();
+        m_availableGeometry = m_virtualDesktop->availableGeometry(m_geometry);
 
     if (m_sizeMillimeters.isEmpty())
         m_sizeMillimeters = virtualDesktop->physicalSize();
 
     m_cursor = new QXcbCursor(connection, this);
 
-    if (connection->hasXRandr()) { // Parse EDID
+    {
+        // Read colord ICC data (from GNOME settings)
+        auto reply = Q_XCB_REPLY_UNCHECKED(xcb_get_property, xcb_connection(),
+                                           false, screen()->root,
+                                           connection->atom(QXcbAtom::_ICC_PROFILE),
+                                           XCB_ATOM_CARDINAL, 0, 8192);
+        if (reply->format == 8 && reply->type == XCB_ATOM_CARDINAL) {
+            QByteArray data(reinterpret_cast<const char *>(xcb_get_property_value(reply.get())), reply->value_len);
+            m_colorSpace = QColorSpace::fromIccProfile(data);
+        }
+    }
+    if (connection->isAtLeastXRandR12()) { // Parse EDID
         QByteArray edid = getEdid();
         if (m_edid.parse(edid)) {
             qCDebug(lcQpaScreen, "EDID data for output \"%s\": identifier '%s', manufacturer '%s',"
@@ -518,6 +579,27 @@ QXcbScreen::QXcbScreen(QXcbConnection *connection, QXcbVirtualDesktop *virtualDe
                     m_edid.model.toLatin1().constData(),
                     m_edid.serialNumber.toLatin1().constData(),
                     m_edid.physicalSize.width(), m_edid.physicalSize.height());
+            if (!m_colorSpace.isValid()) {
+                if (m_edid.sRgb)
+                    m_colorSpace = QColorSpace::SRgb;
+                else {
+                    if (!m_edid.useTables) {
+                        m_colorSpace = QColorSpace(m_edid.whiteChromaticity, m_edid.redChromaticity,
+                                                   m_edid.greenChromaticity, m_edid.blueChromaticity,
+                                                   QColorSpace::TransferFunction::Gamma, m_edid.gamma);
+                    } else {
+                        if (m_edid.tables.length() == 1) {
+                            m_colorSpace = QColorSpace(m_edid.whiteChromaticity, m_edid.redChromaticity,
+                                                       m_edid.greenChromaticity, m_edid.blueChromaticity,
+                                                       m_edid.tables[0]);
+                        } else if (m_edid.tables.length() == 3) {
+                            m_colorSpace = QColorSpace(m_edid.whiteChromaticity, m_edid.redChromaticity,
+                                                       m_edid.greenChromaticity, m_edid.blueChromaticity,
+                                                       m_edid.tables[0], m_edid.tables[1], m_edid.tables[2]);
+                        }
+                    }
+                }
+            }
         } else {
             // This property is defined by the xrandr spec. Parsing failure indicates a valid error,
             // but keep this as debug, for details see 4f515815efc318ddc909a0399b71b8a684962f38.
@@ -525,6 +607,119 @@ QXcbScreen::QXcbScreen(QXcbConnection *connection, QXcbVirtualDesktop *virtualDe
                                     "edid data: " << edid;
         }
     }
+    if (!m_colorSpace.isValid())
+        m_colorSpace = QColorSpace::SRgb;
+}
+
+QXcbScreen::QXcbScreen(QXcbConnection *connection, QXcbVirtualDesktop *virtualDesktop,
+                       xcb_randr_monitor_info_t *monitorInfo, xcb_timestamp_t timestamp)
+    : QXcbObject(connection)
+    , m_virtualDesktop(virtualDesktop)
+    , m_monitor(monitorInfo)
+{
+    setMonitor(monitorInfo, timestamp);
+}
+
+void QXcbScreen::setMonitor(xcb_randr_monitor_info_t *monitorInfo, xcb_timestamp_t timestamp)
+{
+    if (!connection()->isAtLeastXRandR15())
+        return;
+
+    m_outputs.clear();
+    m_crtcs.clear();
+
+    if (!monitorInfo) {
+        m_monitor = nullptr;
+        m_output = XCB_NONE;
+        m_crtc = XCB_NONE;
+        m_mode = XCB_NONE;
+        m_outputName = defaultName();
+        // TODO: Send an event to the QScreen instance that the screen changed its name
+        return;
+    }
+
+    xcb_randr_select_input(xcb_connection(), screen()->root, true);
+
+    m_monitor = monitorInfo;
+    QRect monitorGeometry = QRect(m_monitor->x, m_monitor->y,
+                                  m_monitor->width, m_monitor->height);
+
+    int outputCount = xcb_randr_monitor_info_outputs_length(m_monitor);
+    xcb_randr_output_t *outputs = nullptr;
+    if (outputCount) {
+        outputs = xcb_randr_monitor_info_outputs(m_monitor);
+        for (int i = 0; i < outputCount; i++) {
+            auto output = Q_XCB_REPLY_UNCHECKED(xcb_randr_get_output_info,
+                                                xcb_connection(), outputs[i], timestamp);
+            // Invalid, disconnected or disabled output
+            if (!output)
+                continue;
+
+            if (output->connection != XCB_RANDR_CONNECTION_CONNECTED) {
+                qCDebug(lcQpaScreen, "Output %s is not connected", qPrintable(
+                            QString::fromUtf8((const char*)xcb_randr_get_output_info_name(output.get()),
+                                              xcb_randr_get_output_info_name_length(output.get()))));
+                continue;
+            }
+
+            if (output->crtc == XCB_NONE) {
+                qCDebug(lcQpaScreen, "Output %s is not enabled", qPrintable(
+                            QString::fromUtf8((const char*)xcb_randr_get_output_info_name(output.get()),
+                                              xcb_randr_get_output_info_name_length(output.get()))));
+                continue;
+            }
+
+            m_outputs << outputs[i];
+            m_crtcs << output->crtc;
+        }
+    }
+
+    if (m_crtcs.size() == 1) {
+        auto crtc = Q_XCB_REPLY(xcb_randr_get_crtc_info,
+                                xcb_connection(), m_crtcs[0], timestamp);
+        m_singlescreen = (monitorGeometry == (QRect(crtc->x, crtc->y, crtc->width, crtc->height)));
+        if (m_singlescreen) {
+            if (crtc->mode) {
+                if (crtc->rotation == XCB_RANDR_ROTATION_ROTATE_90 ||
+                    crtc->rotation == XCB_RANDR_ROTATION_ROTATE_270)
+                    std::swap(crtc->width, crtc->height);
+                updateGeometry(QRect(crtc->x, crtc->y, crtc->width, crtc->height), crtc->rotation);
+                if (mode() != crtc->mode)
+                    updateRefreshRate(crtc->mode);
+            }
+        }
+    }
+
+    if (!m_singlescreen)
+        m_geometry = monitorGeometry;
+    m_availableGeometry = m_virtualDesktop->availableGeometry(m_geometry);
+    if (m_geometry.isEmpty())
+        m_geometry = QRect(QPoint(), virtualDesktop()->size());
+    if (m_availableGeometry.isEmpty())
+        m_availableGeometry = m_virtualDesktop->availableGeometry(m_geometry);
+
+    m_sizeMillimeters = sizeInMillimeters(m_geometry.size(), m_virtualDesktop->dpi());
+
+    if (m_sizeMillimeters.isEmpty())
+        m_sizeMillimeters = virtualDesktop()->physicalSize();
+
+    QByteArray ba = connection()->atomName(monitorInfo->name);
+    m_outputName = getName(monitorInfo);
+    m_primary = monitorInfo->primary;
+
+    m_cursor = new QXcbCursor(connection(), this);
+}
+
+QString QXcbScreen::defaultName()
+{
+    QString name;
+    QByteArray displayName = connection()->displayName();
+    int dotPos = displayName.lastIndexOf('.');
+    if (dotPos != -1)
+        displayName.truncate(dotPos);
+    name = QString::fromLocal8Bit(displayName) + QLatin1Char('.')
+            + QString::number(m_virtualDesktop->number());
+    return name;
 }
 
 QXcbScreen::~QXcbScreen()
@@ -538,6 +733,18 @@ QString QXcbScreen::getOutputName(xcb_randr_get_output_info_reply_t *outputInfo)
     if (outputInfo) {
         name = QString::fromUtf8((const char*)xcb_randr_get_output_info_name(outputInfo),
                                  xcb_randr_get_output_info_name_length(outputInfo));
+    } else {
+        name = defaultName();
+    }
+    return name;
+}
+
+QString QXcbScreen::getName(xcb_randr_monitor_info_t *monitorInfo)
+{
+    QString name;
+    QByteArray ba = connection()->atomName(monitorInfo->name);
+    if (!ba.isEmpty()) {
+        name = QString::fromLatin1(ba.constData());
     } else {
         QByteArray displayName = connection()->displayName();
         int dotPos = displayName.lastIndexOf('.');
@@ -577,7 +784,7 @@ QWindow *QXcbScreen::topLevelAt(const QPoint &p) const
     do {
         auto translate_reply = Q_XCB_REPLY_UNCHECKED(xcb_translate_coordinates, xcb_connection(), parent, child, x, y);
         if (!translate_reply) {
-            return 0;
+            return nullptr;
         }
 
         parent = child;
@@ -586,14 +793,14 @@ QWindow *QXcbScreen::topLevelAt(const QPoint &p) const
         y = translate_reply->dst_y;
 
         if (!child || child == root)
-            return 0;
+            return nullptr;
 
         QPlatformWindow *platformWindow = connection()->platformWindowFromId(child);
         if (platformWindow)
             return platformWindow->window();
     } while (parent != child);
 
-    return 0;
+    return nullptr;
 }
 
 void QXcbScreen::windowShown(QXcbWindow *window)
@@ -655,15 +862,13 @@ QImage::Format QXcbScreen::format() const
     bool needsRgbSwap;
     qt_xcb_imageFormatForVisual(connection(), screen()->root_depth, visualForId(screen()->root_visual), &format, &needsRgbSwap);
     // We are ignoring needsRgbSwap here and just assumes the backing-store will handle it.
-    return format;
+    if (format != QImage::Format_Invalid)
+        return format;
+    return QImage::Format_RGB32;
 }
 
 int QXcbScreen::forcedDpi() const
 {
-    static const int overrideDpi = qEnvironmentVariableIntValue("QT_FONT_DPI");
-    if (overrideDpi)
-        return overrideDpi;
-
     const int forcedDpi = m_virtualDesktop->forcedDpi();
     if (forcedDpi > 0)
         return forcedDpi;
@@ -676,12 +881,12 @@ QDpi QXcbScreen::logicalDpi() const
     if (forcedDpi > 0)
         return QDpi(forcedDpi, forcedDpi);
 
-    return m_virtualDesktop->dpi();
-}
-
-qreal QXcbScreen::pixelDensity() const
-{
-    return m_pixelDensity;
+    // Fall back to 96 DPI in case no logical DPI is set. We don't want to
+    // return physical DPI here, since that is a different type of DPI: Logical
+    // DPI typically accounts for user preference and viewing distance, and is
+    // quantized into DPI classes (96, 144, 192, etc); pysical DPI is an exact
+    // physical measure.
+    return QDpi(96, 96);
 }
 
 QPlatformCursor *QXcbScreen::cursor() const
@@ -692,6 +897,7 @@ QPlatformCursor *QXcbScreen::cursor() const
 void QXcbScreen::setOutput(xcb_randr_output_t outputId,
                            xcb_randr_get_output_info_reply_t *outputInfo)
 {
+    m_monitor = nullptr;
     m_output = outputId;
     m_crtc = outputInfo ? outputInfo->crtc : XCB_NONE;
     m_mode = XCB_NONE;
@@ -699,17 +905,9 @@ void QXcbScreen::setOutput(xcb_randr_output_t outputId,
     // TODO: Send an event to the QScreen instance that the screen changed its name
 }
 
-int QXcbScreen::virtualDesktopNumberStatic(const QScreen *screen)
-{
-    if (screen && screen->handle())
-        return static_cast<const QXcbScreen *>(screen->handle())->screenNumber();
-
-    return 0;
-}
-
 void QXcbScreen::updateGeometry(xcb_timestamp_t timestamp)
 {
-    if (!connection()->hasXRandr())
+    if (!connection()->isAtLeastXRandR12())
         return;
 
     auto crtc = Q_XCB_REPLY_UNCHECKED(xcb_randr_get_crtc_info, xcb_connection(),
@@ -747,16 +945,8 @@ void QXcbScreen::updateGeometry(const QRect &geometry, uint8_t rotation)
     if (m_sizeMillimeters.isEmpty())
         m_sizeMillimeters = sizeInMillimeters(geometry.size(), m_virtualDesktop->dpi());
 
-    qreal dpi = forcedDpi();
-    if (dpi <= 0)
-        dpi = geometry.width() / physicalSize().width() * qreal(25.4);
-
-    // Use 128 as a reference DPI on small screens. This favors "small UI" over "large UI".
-    qreal referenceDpi = physicalSize().width() <= 320 ? 128 : 96;
-
-    m_pixelDensity = qMax(1, qRound(dpi/referenceDpi));
     m_geometry = geometry;
-    m_availableGeometry = geometry & m_virtualDesktop->workArea();
+    m_availableGeometry = m_virtualDesktop->availableGeometry(m_geometry);
     QWindowSystemInterface::handleScreenGeometryChange(QPlatformScreen::screen(), m_geometry, m_availableGeometry);
     if (m_orientation != oldOrientation)
         QWindowSystemInterface::handleScreenOrientationChange(QPlatformScreen::screen(), m_orientation);
@@ -764,7 +954,7 @@ void QXcbScreen::updateGeometry(const QRect &geometry, uint8_t rotation)
 
 void QXcbScreen::updateAvailableGeometry()
 {
-    QRect availableGeometry = m_geometry & m_virtualDesktop->workArea();
+    QRect availableGeometry = m_virtualDesktop->availableGeometry(m_geometry);
     if (m_availableGeometry != availableGeometry) {
         m_availableGeometry = availableGeometry;
         QWindowSystemInterface::handleScreenGeometryChange(QPlatformScreen::screen(), m_geometry, m_availableGeometry);
@@ -773,7 +963,7 @@ void QXcbScreen::updateAvailableGeometry()
 
 void QXcbScreen::updateRefreshRate(xcb_randr_mode_t mode)
 {
-    if (!connection()->hasXRandr())
+    if (!connection()->isAtLeastXRandR12())
         return;
 
     if (m_mode == mode)
@@ -909,7 +1099,7 @@ QByteArray QXcbScreen::getOutputProperty(xcb_atom_t atom) const
 QByteArray QXcbScreen::getEdid() const
 {
     QByteArray result;
-    if (!connection()->hasXRandr())
+    if (!connection()->isAtLeastXRandR12())
         return result;
 
     // Try a bunch of atoms
@@ -925,7 +1115,7 @@ QByteArray QXcbScreen::getEdid() const
 static inline void formatRect(QDebug &debug, const QRect r)
 {
     debug << r.width() << 'x' << r.height()
-        << forcesign << r.x() << r.y() << noforcesign;
+        << Qt::forcesign << r.x() << r.y() << Qt::noforcesign;
 }
 
 static inline void formatSizeF(QDebug &debug, const QSizeF s)
@@ -939,7 +1129,7 @@ QDebug operator<<(QDebug debug, const QXcbScreen *screen)
     debug.nospace();
     debug << "QXcbScreen(" << (const void *)screen;
     if (screen) {
-        debug << fixed << qSetRealNumberPrecision(1);
+        debug << Qt::fixed << qSetRealNumberPrecision(1);
         debug << ", name=" << screen->name();
         debug << ", geometry=";
         formatRect(debug, screen->geometry());
@@ -957,7 +1147,7 @@ QDebug operator<<(QDebug debug, const QXcbScreen *screen)
         debug << "), orientation=" << screen->orientation();
         debug << ", depth=" << screen->depth();
         debug << ", refreshRate=" << screen->refreshRate();
-        debug << ", root=" << hex << screen->root();
+        debug << ", root=" << Qt::hex << screen->root();
         debug << ", windowManagerName=" << screen->windowManagerName();
     }
     debug << ')';
