@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2021 The Qt Company Ltd.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
@@ -63,7 +63,7 @@ QSqlTableModelPrivate::~QSqlTableModelPrivate()
 /*! \internal
     Populates our record with values.
 */
-QSqlRecord QSqlTableModelPrivate::record(const QVector<QVariant> &values) const
+QSqlRecord QSqlTableModelPrivate::record(const QList<QVariant> &values) const
 {
     QSqlRecord r = rec;
     for (int i = 0; i < r.count() && i < values.count(); ++i)
@@ -211,7 +211,7 @@ bool QSqlTableModelPrivate::exec(const QString &stmt, bool prepStatement,
     lower-level QSqlQuery and can be used to provide data to view
     classes such as QTableView. For example:
 
-    \snippet sqldatabase/sqldatabase.cpp 24
+    \snippet sqldatabase/sqldatabase_snippet.cpp 24
 
     We set the SQL table's name and the edit strategy, then we set up
     the labels displayed in the view header. The edit strategy
@@ -222,7 +222,7 @@ bool QSqlTableModelPrivate::exec(const QString &stmt, bool prepStatement,
     QSqlTableModel can also be used to access a database
     programmatically, without binding it to a view:
 
-    \snippet sqldatabase/sqldatabase.cpp 21
+    \snippet sqldatabase/sqldatabase.cpp 25
 
     The code snippet above extracts the \c salary field from record 4 in
     the result set of the query \c{SELECT * from employee}.
@@ -291,7 +291,7 @@ bool QSqlTableModelPrivate::exec(const QString &stmt, bool prepStatement,
 
     The default edit strategy is \l OnRowChange.
 */
-QSqlTableModel::QSqlTableModel(QObject *parent, QSqlDatabase db)
+QSqlTableModel::QSqlTableModel(QObject *parent, const QSqlDatabase &db)
     : QSqlQueryModel(*new QSqlTableModelPrivate, parent)
 {
     Q_D(QSqlTableModel);
@@ -300,7 +300,7 @@ QSqlTableModel::QSqlTableModel(QObject *parent, QSqlDatabase db)
 
 /*!  \internal
 */
-QSqlTableModel::QSqlTableModel(QSqlTableModelPrivate &dd, QObject *parent, QSqlDatabase db)
+QSqlTableModel::QSqlTableModel(QSqlTableModelPrivate &dd, QObject *parent, const QSqlDatabase &db)
     : QSqlQueryModel(dd, parent)
 {
     Q_D(QSqlTableModel);
@@ -565,6 +565,10 @@ bool QSqlTableModel::isDirty(const QModelIndex &index) const
     Returns \c true if the value could be set or false on error, for
     example if \a index is out of bounds.
 
+    Returns \c false if the role is not Qt::EditRole. To set data
+    for roles other than EditRole, either use a custom proxy model
+    or subclass QSqlTableModel.
+
     \sa editStrategy(), data(), submit(), submitAll(), revertRow()
 */
 bool QSqlTableModel::setData(const QModelIndex &index, const QVariant &value, int role)
@@ -603,6 +607,14 @@ bool QSqlTableModel::setData(const QModelIndex &index, const QVariant &value, in
     return true;
 }
 
+/*!
+    \reimp
+ */
+bool QSqlTableModel::clearItemData(const QModelIndex &index)
+{
+    return setData(index, QVariant(), Qt::EditRole);
+}
+
 /*!
     This function simply calls QSqlQueryModel::setQuery(\a query).
     You should normally not call it on a QSqlTableModel. Instead, use
@@ -612,7 +624,7 @@ bool QSqlTableModel::setData(const QModelIndex &index, const QVariant &value, in
 */
 void QSqlTableModel::setQuery(const QSqlQuery &query)
 {
-    QSqlQueryModel::setQuery(query);
+    QT_IGNORE_DEPRECATIONS(QSqlQueryModel::setQuery(query);)
 }
 
 /*!
@@ -989,7 +1001,8 @@ QString QSqlTableModel::orderByClause() const
 
     //we can safely escape the field because it would have been obtained from the database
     //and have the correct case
-    QString field = d->tableName + QLatin1Char('.')
+    QString field = d->db.driver()->escapeIdentifier(d->tableName, QSqlDriver::TableName)
+            + QLatin1Char('.')
             + d->db.driver()->escapeIdentifier(f.name(), QSqlDriver::FieldName);
     field = d->sortOrder == Qt::AscendingOrder ? Sql::asc(field) : Sql::desc(field);
     return Sql::orderBy(field);
@@ -1091,12 +1104,8 @@ bool QSqlTableModel::removeColumns(int column, int count, const QModelIndex &par
 bool QSqlTableModel::removeRows(int row, int count, const QModelIndex &parent)
 {
     Q_D(QSqlTableModel);
-    if (parent.isValid() || row < 0 || count <= 0)
+    if (parent.isValid() || row < 0 || count <= 0 || row + count > rowCount())
         return false;
-    else if (row + count > rowCount())
-        return false;
-    else if (!count)
-        return true;
 
     if (d->strategy != OnManualSubmit)
         if (count > 1 || (d->cache.value(row).submitted() && isDirty()))
@@ -1291,7 +1300,7 @@ Qt::ItemFlags QSqlTableModel::flags(const QModelIndex &index) const
     Q_D(const QSqlTableModel);
     if (index.internalPointer() || index.column() < 0 || index.column() >= d->rec.count()
         || index.row() < 0)
-        return 0;
+        return { };
 
     bool editable = true;
 
@@ -1355,7 +1364,7 @@ QSqlRecord QSqlTableModel::record(int row) const
     // get generated flags from the cache
     const QSqlTableModelPrivate::ModifiedRow mrow = d->cache.value(row);
     if (mrow.op() != QSqlTableModelPrivate::None) {
-        const QSqlRecord crec = mrow.rec();
+        const QSqlRecord &crec = mrow.rec();
         for (int i = 0, cnt = rec.count(); i < cnt; ++i)
             rec.setGenerated(i, crec.isGenerated(i));
     }
@@ -1368,12 +1377,12 @@ QSqlRecord QSqlTableModel::record(int row) const
     target fields are mapped by field name, not by position in
     the record.
 
-    Note that the generated flags in \a values are preserved
-    and determine whether the corresponding fields are used when
-    changes are submitted to the database. The caller should
-    remember to set the generated flag to FALSE for fields
-    where the database is meant to supply the value, such as an
-    automatically incremented ID.
+    Note that the generated flags in \a values are preserved to
+    determine whether the corresponding fields are used when changes
+    are submitted to the database. By default, it is set to \c true
+    for all fields in a QSqlRecord. You must set the flag to \c false
+    using \l{QSqlRecord::}{setGenerated}(false) for any value in
+    \a values, to save changes back to the database.
 
     For edit strategies OnFieldChange and OnRowChange, a row may
     receive a change only if no other row has a cached change.
