@@ -40,7 +40,9 @@
 #include "qlineedit.h"
 #include "qlineedit_p.h"
 
-#include "qaction.h"
+#if QT_CONFIG(action)
+#  include "qaction.h"
+#endif
 #include "qapplication.h"
 #include "qclipboard.h"
 #if QT_CONFIG(draganddrop)
@@ -62,7 +64,6 @@
 #include "qtimer.h"
 #include "qvalidator.h"
 #include "qvariant.h"
-#include "qvector.h"
 #include "qdebug.h"
 #if QT_CONFIG(textedit)
 #include "qtextedit.h"
@@ -78,12 +79,11 @@
 #endif
 #include "private/qstylesheetstyle_p.h"
 
-#ifndef QT_NO_SHORTCUT
+#if QT_CONFIG(shortcut)
 #include "private/qapplication_p.h"
 #include "private/qshortcutmap_p.h"
 #include "qkeysequence.h"
-#define ACCEL_KEY(k) ((!QCoreApplication::testAttribute(Qt::AA_DontShowIconsInMenus) \
-                        && QGuiApplication::styleHints()->showShortcutsInContextMenus()) \
+#define ACCEL_KEY(k) (!QCoreApplication::testAttribute(Qt::AA_DontShowShortcutsInContextMenus) \
                       && !QGuiApplicationPrivate::instance()->shortcutMap.hasShortcutForKeySequence(k) ? \
                       QLatin1Char('\t') + QKeySequence(k).toString(QKeySequence::NativeText) : QString())
 #else
@@ -166,7 +166,8 @@ void QLineEdit::initStyleOption(QStyleOptionFrame *option) const
 
     When editing is finished, either because the line edit lost focus
     or Return/Enter is pressed the editingFinished() signal is
-    emitted.
+    emitted. Note that if focus is lost without any changes done,
+    the editingFinished() signal won't be emitted.
 
     Note that if there is a validator set on the line edit, the
     returnPressed()/editingFinished() signals will only be emitted if
@@ -207,7 +208,7 @@ void QLineEdit::initStyleOption(QStyleOptionFrame *option) const
     Any other key sequence that represents a valid character, will
     cause the character to be inserted into the line edit.
 
-    \sa QTextEdit, QLabel, QComboBox, {fowler}{GUI Design Handbook: Field, Entry}, {Line Edits Example}
+    \sa QTextEdit, QLabel, QComboBox, {Line Edits Example}
 */
 
 
@@ -274,7 +275,7 @@ QLineEdit::QLineEdit(QWidget* parent)
     \sa text(), setMaxLength()
 */
 QLineEdit::QLineEdit(const QString& contents, QWidget* parent)
-    : QWidget(*new QLineEditPrivate, parent, 0)
+    : QWidget(*new QLineEditPrivate, parent, { })
 {
     Q_D(QLineEdit);
     d->init(contents);
@@ -446,7 +447,7 @@ void QLineEdit::addAction(QAction *action, ActionPosition position)
 {
     Q_D(QLineEdit);
     QWidget::addAction(action);
-    d->addAction(action, 0, position);
+    d->addAction(action, nullptr, position);
 }
 
 /*!
@@ -599,15 +600,23 @@ const QValidator * QLineEdit::validator() const
 }
 
 /*!
-    Sets this line edit to only accept input that the validator, \a v,
-    will accept. This allows you to place any arbitrary constraints on
-    the text which may be entered.
+    Sets the validator for values of line edit to \a v.
+
+    The line edit's returnPressed() and editingFinished() signals will only
+    be emitted if \a v validates the line edit's content as \l{QValidator::}{Acceptable}.
+    The user may change the content to any \l{QValidator::}{Intermediate}
+    value during editing, but will be prevented from editing the text to a
+    value that \a v validates as \l{QValidator::}{Invalid}.
+
+    This allows you to constrain the text that shall finally be entered when editing is
+    done, while leaving users with enough freedom to edit the text from one valid state
+    to another.
 
     If \a v == 0, setValidator() removes the current input validator.
     The initial setting is to have no input validator (i.e. any input
     is accepted up to maxLength()).
 
-    \sa validator(), hasAcceptableInput(), QIntValidator, QDoubleValidator, QRegExpValidator
+    \sa validator(), hasAcceptableInput(), QIntValidator, QDoubleValidator, QRegularExpressionValidator
 */
 
 void QLineEdit::setValidator(const QValidator *v)
@@ -640,15 +649,15 @@ void QLineEdit::setCompleter(QCompleter *c)
     if (c == d->control->completer())
         return;
     if (d->control->completer()) {
-        disconnect(d->control->completer(), 0, this, 0);
-        d->control->completer()->setWidget(0);
+        disconnect(d->control->completer(), nullptr, this, nullptr);
+        d->control->completer()->setWidget(nullptr);
         if (d->control->completer()->parent() == this)
             delete d->control->completer();
     }
     d->control->setCompleter(c);
     if (!c)
         return;
-    if (c->widget() == 0)
+    if (c->widget() == nullptr)
         c->setWidget(this);
     if (hasFocus()) {
         QObject::connect(d->control->completer(), SIGNAL(activated(QString)),
@@ -683,7 +692,7 @@ QSize QLineEdit::sizeHint() const
     Q_D(const QLineEdit);
     ensurePolished();
     QFontMetrics fm(font());
-    const int iconSize = style()->pixelMetric(QStyle::PM_SmallIconSize, 0, this);
+    const int iconSize = style()->pixelMetric(QStyle::PM_SmallIconSize, nullptr, this);
     const QMargins tm = d->effectiveTextMargins();
     int h = qMax(fm.height(), qMax(14, iconSize - 2)) + 2 * QLineEditPrivate::verticalMargin
             + tm.top() + tm.bottom()
@@ -693,15 +702,14 @@ QSize QLineEdit::sizeHint() const
             + d->leftmargin + d->rightmargin; // "some"
     QStyleOptionFrame opt;
     initStyleOption(&opt);
-    return (style()->sizeFromContents(QStyle::CT_LineEdit, &opt, QSize(w, h).
-                                      expandedTo(QApplication::globalStrut()), this));
+    return style()->sizeFromContents(QStyle::CT_LineEdit, &opt, QSize(w, h), this);
 }
 
 
 /*!
     Returns a minimum size for the line edit.
 
-    The width returned is enough for at least one character.
+    The width returned is usually enough for at least one character.
 */
 
 QSize QLineEdit::minimumSizeHint() const
@@ -713,13 +721,12 @@ QSize QLineEdit::minimumSizeHint() const
     int h = fm.height() + qMax(2 * QLineEditPrivate::verticalMargin, fm.leading())
             + tm.top() + tm.bottom()
             + d->topmargin + d->bottommargin;
-    int w = fm.maxWidth()
+    int w = fm.maxWidth() + 2 * QLineEditPrivate::horizontalMargin
             + tm.left() + tm.right()
             + d->leftmargin + d->rightmargin;
     QStyleOptionFrame opt;
     initStyleOption(&opt);
-    return (style()->sizeFromContents(QStyle::CT_LineEdit, &opt, QSize(w, h).
-                                      expandedTo(QApplication::globalStrut()), this));
+    return style()->sizeFromContents(QStyle::CT_LineEdit, &opt, QSize(w, h), this);
 }
 
 
@@ -1150,30 +1157,6 @@ void QLineEdit::setTextMargins(const QMargins &margins)
     update();
 }
 
-#if QT_DEPRECATED_SINCE(5, 14)
-/*!
-    \obsolete
-    Use textMargins()
-
-    Returns the widget's text margins for \a left, \a top, \a right, and \a bottom.
-    \since 4.5
-
-    \sa setTextMargins()
-*/
-void QLineEdit::getTextMargins(int *left, int *top, int *right, int *bottom) const
-{
-    QMargins m = textMargins();
-    if (left)
-        *left = m.left();
-    if (top)
-        *top = m.top();
-    if (right)
-        *right = m.right();
-    if (bottom)
-        *bottom = m.bottom();
-}
-#endif
-
 /*!
     \since 4.6
     Returns the widget's text margins.
@@ -1198,40 +1181,59 @@ QMargins QLineEdit::textMargins() const
     Unset the mask and return to normal QLineEdit operation by passing
     an empty string ("").
 
-    The table below shows the characters that can be used in an input mask.
-    A space character, the default character for a blank, is needed for cases
-    where a character is \e{permitted but not required}.
+    The input mask is an input template string. It can contain the following elements:
+    \table
+    \row \li Mask Characters \li Defines the \l {QChar::} {Category} of input characters
+    that are considered valid in this position
+    \row \li Meta Characters \li Various special meanings
+    \row \li Separators \li All other characters are regarded as immutable separators
+    \endtable
+
+    The following table shows the mask and meta characters that can be used in an input mask.
 
     \table
-    \header \li Character \li Meaning
-    \row \li \c A \li ASCII alphabetic character required. A-Z, a-z.
-    \row \li \c a \li ASCII alphabetic character permitted but not required.
-    \row \li \c N \li ASCII alphanumeric character required. A-Z, a-z, 0-9.
-    \row \li \c n \li ASCII alphanumeric character permitted but not required.
-    \row \li \c X \li Any character required.
-    \row \li \c x \li Any character permitted but not required.
-    \row \li \c 9 \li ASCII digit required. 0-9.
-    \row \li \c 0 \li ASCII digit permitted but not required.
-    \row \li \c D \li ASCII digit required. 1-9.
-    \row \li \c d \li ASCII digit permitted but not required (1-9).
-    \row \li \c # \li ASCII digit or plus/minus sign permitted but not required.
+    \header \li Mask Character \li Meaning
+    \row \li \c A \li character of the Letter category required, such as A-Z, a-z.
+    \row \li \c a \li character of the Letter category permitted but not required.
+    \row \li \c N \li character of the Letter or Number category required, such as
+                      A-Z, a-z, 0-9.
+    \row \li \c n \li character of the Letter or Number category permitted but not required.
+    \row \li \c X \li Any non-blank character required.
+    \row \li \c x \li Any non-blank character permitted but not required.
+    \row \li \c 9 \li character of the Number category required, e.g 0-9.
+    \row \li \c 0 \li character of the Number category permitted but not required.
+    \row \li \c D \li character of the Number category and larger than zero required,
+                      such as 1-9
+    \row \li \c d \li character of the Number category and larger than zero permitted but not
+                      required, such as 1-9.
+    \row \li \c # \li character of the Number category, or plus/minus sign permitted but not
+                      required.
     \row \li \c H \li Hexadecimal character required. A-F, a-f, 0-9.
     \row \li \c h \li Hexadecimal character permitted but not required.
     \row \li \c B \li Binary character required. 0-1.
     \row \li \c b \li Binary character permitted but not required.
+    \header \li Meta Character \li Meaning
     \row \li \c > \li All following alphabetic characters are uppercased.
     \row \li \c < \li All following alphabetic characters are lowercased.
     \row \li \c ! \li Switch off case conversion.
+    \row \li \c {;c} \li Terminates the input mask and sets the \e{blank} character to \e{c}.
     \row \li \c {[ ] { }} \li Reserved.
     \row \li \tt{\\} \li Use \tt{\\} to escape the special
                            characters listed above to use them as
                            separators.
     \endtable
 
-    The mask consists of a string of mask characters and separators,
-    optionally followed by a semicolon and the character used for
-    blanks. The blank characters are always removed from the text
-    after editing.
+    When created or cleared, the line edit will be filled with a copy of the
+    input mask string where the meta characters have been removed, and the mask
+    characters have been replaced with the \e{blank} character (by default, a
+    \c space).
+
+    When an input mask is set, the text() method returns a modified copy of the
+    line edit content where all the \e{blank} characters have been removed. The
+    unmodified content can be read using displayText().
+
+    The hasAcceptableInput() method returns false if the current content of the
+    line edit does not fulfil the requirements of the input mask.
 
     Examples:
     \table
@@ -1240,14 +1242,14 @@ QMargins QLineEdit::textMargins() const
     \row \li \c HH:HH:HH:HH:HH:HH;_ \li MAC address
     \row \li \c 0000-00-00 \li ISO Date; blanks are \c space
     \row \li \c >AAAAA-AAAAA-AAAAA-AAAAA-AAAAA;# \li License number;
-    blanks are \c - and all (alphabetic) characters are converted to
+    blanks are \c{#} and all (alphabetic) characters are converted to
     uppercase.
     \endtable
 
     To get range control (e.g., for an IP address) use masks together
     with \l{setValidator()}{validators}.
 
-    \sa maxLength
+    \sa maxLength, QChar::isLetter(), QChar::isNumber(), QChar::digitValue()
 */
 QString QLineEdit::inputMask() const
 {
@@ -1420,7 +1422,7 @@ void QLineEdit::copy() const
     Inserts the clipboard's text at the cursor position, deleting any
     selected text, providing the line edit is not \l{QLineEdit::readOnly}{read-only}.
 
-    If the end result would not be acceptable to the current
+    If the end result would be invalid to the current
     \l{setValidator()}{validator}, nothing happens.
 
     \sa copy(), cut()
@@ -1434,23 +1436,29 @@ void QLineEdit::paste()
 
 #endif // !QT_NO_CLIPBOARD
 
+/*!
+    \reimp
+*/
+void QLineEdit::timerEvent(QTimerEvent *e)
+{
+    Q_D(QLineEdit);
+    int timerId = ((QTimerEvent*)e)->timerId();
+    if (false) {
+#if QT_CONFIG(draganddrop)
+    } else if (timerId == d->dndTimer.timerId()) {
+        d->drag();
+#endif
+    }
+    else if (timerId == d->tripleClickTimer.timerId())
+        d->tripleClickTimer.stop();
+}
+
 /*! \reimp
 */
 bool QLineEdit::event(QEvent * e)
 {
     Q_D(QLineEdit);
-    if (e->type() == QEvent::Timer) {
-        // ### Qt6: move to timerEvent, is here for binary compatibility
-        int timerId = ((QTimerEvent*)e)->timerId();
-        if (false) {
-#if QT_CONFIG(draganddrop)
-        } else if (timerId == d->dndTimer.timerId()) {
-            d->drag();
-#endif
-        }
-        else if (timerId == d->tripleClickTimer.timerId())
-            d->tripleClickTimer.stop();
-    } else if (e->type() == QEvent::ContextMenu) {
+    if (e->type() == QEvent::ContextMenu) {
 #ifndef QT_NO_IM
         if (d->control->composeMode())
             return true;
@@ -1463,8 +1471,6 @@ bool QLineEdit::event(QEvent * e)
         QKeyEvent *ke = static_cast<QKeyEvent*>(e);
         d->control->processShortcutOverrideEvent(ke);
 #endif
-    } else if (e->type() == QEvent::KeyRelease) {
-        d->control->updateCursorBlinking();
     } else if (e->type() == QEvent::Show) {
         //In order to get the cursor blinking if QComboBox::setEditable is called when the combobox has focus
         if (hasFocus()) {
@@ -1510,7 +1516,7 @@ void QLineEdit::mousePressEvent(QMouseEvent* e)
 {
     Q_D(QLineEdit);
 
-    d->mousePressPos = e->pos();
+    d->mousePressPos = e->position().toPoint();
 
     if (d->sendMouseEventToInputContext(e))
         return;
@@ -1524,7 +1530,7 @@ void QLineEdit::mousePressEvent(QMouseEvent* e)
             d->control->completer()->complete();
     }
 #endif
-    if (d->tripleClickTimer.isActive() && (e->pos() - d->tripleClick).manhattanLength() <
+    if (d->tripleClickTimer.isActive() && (e->position().toPoint() - d->tripleClick).manhattanLength() <
          QApplication::startDragDistance()) {
         selectAll();
         return;
@@ -1533,10 +1539,10 @@ void QLineEdit::mousePressEvent(QMouseEvent* e)
 #ifdef Q_OS_ANDROID
     mark = mark && (d->imHints & Qt::ImhNoPredictiveText);
 #endif // Q_OS_ANDROID
-    int cursor = d->xToPos(e->pos().x());
+    int cursor = d->xToPos(e->position().toPoint().x());
 #if QT_CONFIG(draganddrop)
     if (!mark && d->dragEnabled && d->control->echoMode() == Normal &&
-         e->button() == Qt::LeftButton && d->inSelection(e->pos().x())) {
+         e->button() == Qt::LeftButton && d->inSelection(e->position().toPoint().x())) {
         if (!d->dndTimer.isActive())
             d->dndTimer.start(QApplication::startDragTime(), this);
     } else
@@ -1555,7 +1561,7 @@ void QLineEdit::mouseMoveEvent(QMouseEvent * e)
     if (e->buttons() & Qt::LeftButton) {
 #if QT_CONFIG(draganddrop)
         if (d->dndTimer.isActive()) {
-            if ((d->mousePressPos - e->pos()).manhattanLength() > QApplication::startDragDistance())
+            if ((d->mousePressPos - e->position().toPoint()).manhattanLength() > QApplication::startDragDistance())
                 d->drag();
         } else
 #endif
@@ -1566,26 +1572,26 @@ void QLineEdit::mouseMoveEvent(QMouseEvent * e)
             const bool select = (d->imHints & Qt::ImhNoPredictiveText);
 #endif
 #ifndef QT_NO_IM
-            if (d->mouseYThreshold > 0 && e->pos().y() > d->mousePressPos.y() + d->mouseYThreshold) {
+            if (d->mouseYThreshold > 0 && e->position().toPoint().y() > d->mousePressPos.y() + d->mouseYThreshold) {
                 if (layoutDirection() == Qt::RightToLeft)
                     d->control->home(select);
                 else
                     d->control->end(select);
-            } else if (d->mouseYThreshold > 0 && e->pos().y() + d->mouseYThreshold < d->mousePressPos.y()) {
+            } else if (d->mouseYThreshold > 0 && e->position().toPoint().y() + d->mouseYThreshold < d->mousePressPos.y()) {
                 if (layoutDirection() == Qt::RightToLeft)
                     d->control->end(select);
                 else
                     d->control->home(select);
             } else if (d->control->composeMode() && select) {
                 int startPos = d->xToPos(d->mousePressPos.x());
-                int currentPos = d->xToPos(e->pos().x());
+                int currentPos = d->xToPos(e->position().toPoint().x());
                 if (startPos != currentPos)
                     d->control->setSelection(startPos, currentPos - startPos);
 
             } else
 #endif
             {
-                d->control->moveCursor(d->xToPos(e->pos().x()), select);
+                d->control->moveCursor(d->xToPos(e->position().toPoint().x()), select);
             }
         }
     }
@@ -1613,14 +1619,14 @@ void QLineEdit::mouseReleaseEvent(QMouseEvent* e)
     if (QGuiApplication::clipboard()->supportsSelection()) {
         if (e->button() == Qt::LeftButton) {
             d->control->copy(QClipboard::Selection);
-        } else if (!d->control->isReadOnly() && e->button() == Qt::MidButton) {
+        } else if (!d->control->isReadOnly() && e->button() == Qt::MiddleButton) {
             deselect();
             d->control->paste(QClipboard::Selection);
         }
     }
 #endif
 
-    if (!isReadOnly() && rect().contains(e->pos()))
+    if (!isReadOnly() && rect().contains(e->position().toPoint()))
         d->handleSoftwareInputPanel(e->button(), d->clickCausedFocus);
     d->clickCausedFocus = 0;
 }
@@ -1632,7 +1638,7 @@ void QLineEdit::mouseDoubleClickEvent(QMouseEvent* e)
     Q_D(QLineEdit);
 
     if (e->button() == Qt::LeftButton) {
-        int position = d->xToPos(e->pos().x());
+        int position = d->xToPos(e->position().toPoint().x());
 
         // exit composition mode
 #ifndef QT_NO_IM
@@ -1666,7 +1672,7 @@ void QLineEdit::mouseDoubleClickEvent(QMouseEvent* e)
             d->control->selectWordAtPos(position);
 
         d->tripleClickTimer.start(QApplication::doubleClickInterval(), this);
-        d->tripleClick = e->pos();
+        d->tripleClick = e->position().toPoint();
     } else {
         d->sendMouseEventToInputContext(e);
     }
@@ -1685,8 +1691,11 @@ void QLineEdit::mouseDoubleClickEvent(QMouseEvent* e)
 /*!
     \fn void  QLineEdit::editingFinished()
 
-    This signal is emitted when the Return or Enter key is pressed or
-    the line edit loses focus. Note that if there is a validator() or
+    This signal is emitted when the Return or Enter key is pressed, or
+    if the line edit loses focus and its contents have changed since the
+    last time this signal was emitted.
+
+    Note that if there is a validator() or
     inputMask() set on the line edit and enter/return is pressed, the
     editingFinished() signal will only be emitted if the input follows
     the inputMask() and the validator() returns QValidator::Acceptable.
@@ -1765,11 +1774,19 @@ void QLineEdit::keyPressEvent(QKeyEvent *event)
     }
 #endif
     d->control->processKeyEvent(event);
-    if (event->isAccepted()) {
-        if (layoutDirection() != d->control->layoutDirection())
-            setLayoutDirection(d->control->layoutDirection());
+    if (event->isAccepted())
         d->control->updateCursorBlinking();
-    }
+}
+
+/*!
+    \reimp
+*/
+void QLineEdit::keyReleaseEvent(QKeyEvent *)
+{
+    Q_D(QLineEdit);
+    if (!isReadOnly())
+        d->handleSoftwareInputPanel();
+    d->control->updateCursorBlinking();
 }
 
 /*!
@@ -1788,10 +1805,6 @@ QRect QLineEdit::cursorRect() const
 void QLineEdit::inputMethodEvent(QInputMethodEvent *e)
 {
     Q_D(QLineEdit);
-    if (d->control->isReadOnly()) {
-        e->ignore();
-        return;
-    }
 
     if (echoMode() == PasswordEchoOnEdit && !d->control->passwordEchoEditing()) {
         // Clear the edit and reset to normal echo mode while entering input
@@ -1838,6 +1851,7 @@ QVariant QLineEdit::inputMethodQuery(Qt::InputMethodQuery property, QVariant arg
         return d->adjustedControlRect(d->control->anchorRect());
     case Qt::ImFont:
         return font();
+    case Qt::ImAbsolutePosition:
     case Qt::ImCursorPosition: {
         const QPointF pt = argument.toPointF();
         if (!pt.isNull())
@@ -1856,6 +1870,20 @@ QVariant QLineEdit::inputMethodQuery(Qt::InputMethodQuery property, QVariant arg
             return QVariant(d->control->selectionEnd());
         else
             return QVariant(d->control->selectionStart());
+    case Qt::ImReadOnly:
+        return isReadOnly();
+    case Qt::ImTextBeforeCursor: {
+        const QPointF pt = argument.toPointF();
+        if (!pt.isNull())
+            return d->textBeforeCursor(d->xToPos(pt.x(), QTextLine::CursorBetweenCharacters));
+        else
+            return d->textBeforeCursor(d->control->cursor()); }
+    case Qt::ImTextAfterCursor: {
+        const QPointF pt = argument.toPointF();
+        if (!pt.isNull())
+            return d->textAfterCursor(d->xToPos(pt.x(), QTextLine::CursorBetweenCharacters));
+        else
+            return d->textAfterCursor(d->control->cursor()); }
     default:
         return QWidget::inputMethodQuery(property);
     }
@@ -1874,8 +1902,11 @@ void QLineEdit::focusInEvent(QFocusEvent *e)
             d->control->moveCursor(d->control->nextMaskBlank(0));
         else if (!d->control->hasSelectedText())
             selectAll();
+        else
+            updateMicroFocus();
     } else if (e->reason() == Qt::MouseFocusReason) {
         d->clickCausedFocus = 1;
+        updateMicroFocus();
     }
 #ifdef QT_KEYPAD_NAVIGATION
     if (!QApplicationPrivate::keypadNavigationEnabled() || (hasEditFocus() && ( e->reason() == Qt::PopupFocusReason))) {
@@ -1883,7 +1914,7 @@ void QLineEdit::focusInEvent(QFocusEvent *e)
     d->control->setBlinkingCursorEnabled(true);
     QStyleOptionFrame opt;
     initStyleOption(&opt);
-    if((!hasSelectedText() && d->control->preeditAreaText().isEmpty())
+    if ((!hasSelectedText() && d->control->preeditAreaText().isEmpty())
        || style()->styleHint(QStyle::SH_BlinkCursorWhenTextSelected, &opt, this))
         d->setCursorVisible(true);
 #ifdef QT_KEYPAD_NAVIGATION
@@ -1936,7 +1967,7 @@ void QLineEdit::focusOutEvent(QFocusEvent *e)
 #endif
 #if QT_CONFIG(completer)
     if (d->control->completer()) {
-        QObject::disconnect(d->control->completer(), 0, this, 0);
+        QObject::disconnect(d->control->completer(), nullptr, this, nullptr);
     }
 #endif
     QWidget::focusOutEvent(e);
@@ -1958,21 +1989,28 @@ void QLineEdit::paintEvent(QPaintEvent *)
     p.setClipRect(r);
 
     QFontMetrics fm = fontMetrics();
+    int fmHeight = 0;
+    if (d->shouldShowPlaceholderText())
+        fmHeight = fm.boundingRect(d->placeholderText).height();
+    else
+        fmHeight = fm.boundingRect(d->control->text() + d->control->preeditAreaText()).height();
+    fmHeight = qMax(fmHeight, fm.height());
+
     Qt::Alignment va = QStyle::visualAlignment(d->control->layoutDirection(), QFlag(d->alignment));
     switch (va & Qt::AlignVertical_Mask) {
      case Qt::AlignBottom:
-         d->vscroll = r.y() + r.height() - fm.height() - QLineEditPrivate::verticalMargin;
+         d->vscroll = r.y() + r.height() - fmHeight - QLineEditPrivate::verticalMargin;
          break;
      case Qt::AlignTop:
          d->vscroll = r.y() + QLineEditPrivate::verticalMargin;
          break;
      default:
          //center
-         d->vscroll = r.y() + (r.height() - fm.height() + 1) / 2;
+         d->vscroll = r.y() + (r.height() - fmHeight + 1) / 2;
          break;
     }
     QRect lineRect(r.x() + QLineEditPrivate::horizontalMargin, d->vscroll,
-                   r.width() - 2 * QLineEditPrivate::horizontalMargin, fm.height());
+                   r.width() - 2 * QLineEditPrivate::horizontalMargin, fmHeight);
 
     if (d->shouldShowPlaceholderText()) {
         if (!d->placeholderText.isEmpty()) {
@@ -2029,6 +2067,7 @@ void QLineEdit::paintEvent(QPaintEvent *)
     }
 
     // the y offset is there to keep the baseline constant in case we have script changes in the text.
+    // Needs to be kept in sync with QLineEditPrivate::adjustedControlRect
     QPoint topLeft = lineRect.topLeft() - QPoint(d->hscroll, d->control->ascent() - fm.ascent());
 
     // draw text, selections and cursors
@@ -2054,11 +2093,13 @@ void QLineEdit::paintEvent(QPaintEvent *)
 
     // Asian users see an IM selection text as cursor on candidate
     // selection phase of input method, so the ordinary cursor should be
-    // invisible if we have a preedit string.
-    if (d->cursorVisible && !d->control->isReadOnly())
+    // invisible if we have a preedit string. another condition is when inputmask
+    // isn't empty,we don't need draw cursor,because cursor and character overlapping
+    // area is white.
+    if (d->cursorVisible && !d->control->isReadOnly() && d->control->inputMask().isEmpty())
         flags |= QWidgetLineControl::DrawCursor;
 
-    d->control->setCursorWidth(style()->pixelMetric(QStyle::PM_TextCursorWidth));
+    d->control->setCursorWidth(style()->pixelMetric(QStyle::PM_TextCursorWidth, &panel));
     d->control->draw(&p, topLeft, r, flags);
 
 }
@@ -2072,7 +2113,7 @@ void QLineEdit::dragMoveEvent(QDragMoveEvent *e)
     Q_D(QLineEdit);
     if (!d->control->isReadOnly() && e->mimeData()->hasFormat(QLatin1String("text/plain"))) {
         e->acceptProposedAction();
-        d->control->moveCursor(d->xToPos(e->pos().x()), false);
+        d->control->moveCursor(d->xToPos(e->position().toPoint().x()), false);
         d->cursorVisible = true;
         update();
     }
@@ -2103,7 +2144,7 @@ void QLineEdit::dropEvent(QDropEvent* e)
     if (!str.isNull() && !d->control->isReadOnly()) {
         if (e->source() == this && e->dropAction() == Qt::CopyAction)
             deselect();
-        int cursorPos = d->xToPos(e->pos().x());
+        int cursorPos = d->xToPos(e->position().toPoint().x());
         int selStart = cursorPos;
         int oldSelStart = d->control->selectionStart();
         int oldSelEnd = d->control->selectionEnd();
@@ -2175,7 +2216,7 @@ QMenu *QLineEdit::createStandardContextMenu()
     Q_D(QLineEdit);
     QMenu *popup = new QMenu(this);
     popup->setObjectName(QLatin1String("qt_edit_menu"));
-    QAction *action = 0;
+    QAction *action = nullptr;
 
     if (!isReadOnly()) {
         action = popup->addAction(QLineEdit::tr("&Undo") + ACCEL_KEY(QKeySequence::Undo));
@@ -2256,7 +2297,7 @@ void QLineEdit::changeEvent(QEvent *ev)
         {
             QStyleOptionFrame opt;
             initStyleOption(&opt);
-            d->control->setPasswordCharacter(style()->styleHint(QStyle::SH_LineEdit_PasswordCharacter, &opt, this));
+            d->control->setPasswordCharacter(char16_t(style()->styleHint(QStyle::SH_LineEdit_PasswordCharacter, &opt, this)));
             d->control->setPasswordMaskDelay(style()->styleHint(QStyle::SH_LineEdit_PasswordMaskDelay, &opt, this));
         }
         update();
