@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2020 The Qt Company Ltd.
+** Copyright (C) 2021 The Qt Company Ltd.
 ** Copyright (C) 2016 Intel Corporation.
 ** Contact: https://www.qt.io/licensing/
 **
@@ -29,7 +29,7 @@
 
 #include <QtCore/QCoreApplication>
 
-#if QT_CONFIG(process)
+QT_REQUIRE_CONFIG(process);
 
 #if QT_CONFIG(temporaryfile)
 #  define USE_DIFF
@@ -220,6 +220,8 @@ bool compareOutput(const QString &logger, const QString &subdir,
             continue;
         if (actualLineBA.endsWith(" : failure location"))
             continue;
+        if (actualLineBA.endsWith(" : message location"))
+            continue;
 
         if (actualLineBA.startsWith("Config: Using QtTest library") // Text build string
             || actualLineBA.startsWith("    <QtBuild") // XML, Light XML build string
@@ -236,6 +238,8 @@ bool compareOutput(const QString &logger, const QString &subdir,
             actualLine.replace(timestampRegex, "timestamp=\"@TEST_START_TIME@\"");
             static QRegularExpression timeRegex("time=\".*?\"");
             actualLine.replace(timeRegex, "time=\"@TEST_DURATION@\"");
+            static QRegularExpression hostnameRegex("hostname=\".*?\"");
+            actualLine.replace(hostnameRegex, "hostname=\"@HOSTNAME@\"");
         }
 
         // Special handling for ignoring _FILE_ and _LINE_ if logger is teamcity
@@ -602,10 +606,9 @@ struct TestLogger
 
     QString outputFileName(const QString &test) const
     {
-        if (outputMode == StdoutOutput)
-            return QString();
-
-        return testOutputDir.filePath("output_" + test + "." + shortName());
+        return testOutputDir.filePath("output_" + test +
+            (outputMode == StdoutOutput ? ".stdout" : "") +
+            "." + shortName());
     }
 
     QString expectationFileName(const QString &test, int version = 0) const
@@ -619,16 +622,15 @@ struct TestLogger
 
     QStringList arguments(const QString &test) const
     {
-        auto fileName = outputFileName(test);
-
         QStringList arguments;
         if (argumentStyle == NewStyleArgument) {
-            arguments << "-o" << (!fileName.isEmpty() ? fileName : QStringLiteral("-"))
-                + "," + shortName();
+            arguments << "-o" << (outputMode == FileOutput
+                ? outputFileName(test) : QStringLiteral("-")) +
+                "," + shortName();
         } else {
             arguments << "-" + shortName();
-            if (!fileName.isEmpty())
-                arguments << "-o" << fileName;
+            if (outputMode == FileOutput)
+                arguments << "-o" << outputFileName(test);
         }
 
         return arguments;
@@ -636,9 +638,6 @@ struct TestLogger
 
     QByteArray testOutput(const QString &test) const
     {
-        if (outputMode == StdoutOutput)
-            return QByteArray();
-
         QFile outputFile(outputFileName(test));
         REQUIRE(outputFile.exists());
         REQUIRE(outputFile.open(QIODevice::ReadOnly));
@@ -661,9 +660,6 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
         return true;
 #endif
 
-    if (test == "deleteLater" || test == "deleteLater_noApp" || test == "mouse")
-        return true; // Missing expectation files
-
     // These tests are affected by timing and whether the CPU tick counter
     // is monotonically increasing. They won't work on some machines so
     // leave them off by default. Feel free to enable them for your own
@@ -695,17 +691,18 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
 #endif
 
     if (test == "benchlibcallgrind") {
-#if !(defined(__GNUC__) && defined(__i386) && defined(Q_OS_LINUX))
-        // Skip on platforms where callgrind is not available
-        return true;
-#else
+#if defined(__GNUC__) && (defined(__i386) || defined(__x86_64)) && defined(Q_OS_LINUX)
         // Check that it's actually available
         QProcess checkProcess;
-        QStringList args;
-        args << "--version";
+        QStringList args{u"--version"_qs};
         checkProcess.start("valgrind", args);
-        if (!checkProcess.waitForFinished(-1))
+        if (!checkProcess.waitForFinished(-1)) {
             WARN("Valgrind broken or not available. Not running benchlibcallgrind test!");
+            return true;
+        }
+#else
+        // Skip on platforms where callgrind is not available
+        return true;
 #endif
     }
 
@@ -722,8 +719,11 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
             || test == "silent")
             return true;
 
-        // `crashes' will not output valid XML on platforms without a crash handler
-        if (test == "crashes")
+        // These tests produce variable output (callgrind because of #if-ery,
+        // crashes by virtue of platform differences in where the output cuts
+        // off), so only test them for one format, to avoid the need for several
+        // _n variants for each format. Also, crashes can produce invalid XML.
+        if (test == "crashes" || test == "benchlibcallgrind")
             return true;
 
         // this test prints out some floats in the testlog and the formatting is
@@ -731,10 +731,11 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
         if (test == "float")
             return true;
 
-        // these tests are quite slow, and running them for all the loggers significantly
-        // increases the overall test time.  They do not really relate to logging, so it
-        // should be safe to run them just for the stdout loggers.
-        if (test == "benchlibcallgrind" || test == "sleep")
+        // This test is quite slow, and running it for all the loggers
+        // significantly increases the overall test time.  It does not really
+        // relate to logging, so it should be safe to run it just for the stdout
+        // loggers.
+        if (test == "sleep")
             return true;
     }
 
@@ -742,11 +743,12 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
             || logger == QTestLog::LightXML || logger == QTestLog::JUnitXML))
         return true;
 
-    if (logger == QTestLog::CSV && !test.startsWith("benchlib"))
+    // Skip benchmark for TeamCity logger, skip everything else for CSV:
+    if (logger == (test.startsWith("benchlib") ? QTestLog::TeamCity : QTestLog::CSV))
         return true;
 
-    if (logger == QTestLog::TeamCity && test.startsWith("benchlib"))
-        return true; // Skip benchmark for TeamCity logger
+    if (logger != QTestLog::JUnitXML && test == "junit")
+        return true;
 
     return false;
 }
@@ -775,7 +777,7 @@ void checkErrorOutput(const QString &test, const QByteArray &errorOutput)
         || test == "cmptest" // QImage comparison requires QGuiApplication
         || test == "fetchbogus"
         || test == "watchdog"
-        || test == "xunit"
+        || test == "junit"
         || test == "benchlibcallgrind")
         return;
 
@@ -785,6 +787,11 @@ void checkErrorOutput(const QString &test, const QByteArray &errorOutput)
 #endif
         return;
 
+#ifdef Q_OS_WIN
+    if (test == "crashes")
+        return; // Complains about uncaught exception
+#endif
+
 #ifdef Q_OS_LINUX
     // QEMU outputs to stderr about uncaught signals
     if (QTestPrivate::isRunningArmOnX86() &&
@@ -1047,10 +1054,14 @@ void runTest(const QString &test, const TestLoggers &requestedLoggers)
 
     for (auto logger : loggers) {
         QByteArray testOutput;
-        if (logger.outputMode == StdoutOutput)
+        if (logger.outputMode == StdoutOutput) {
             testOutput = testProcess.standardOutput;
-        else
+            QFile file(logger.outputFileName(test));
+            REQUIRE(file.open(QIODevice::WriteOnly));
+            file.write(testOutput);
+        } else {
             testOutput = logger.testOutput(test);
+        }
 
         checkTestOutput(test, logger, testOutput);
     }
@@ -1202,21 +1213,16 @@ SCENARIO("Test output of the loggers is as expected")
     GIVEN("The " << logger << " logger") {
         for (QString test : tests) {
             AND_GIVEN("The " << test << " subtest") {
-                runTest(test, TestLogger(logger));
+                runTest(test, TestLogger(logger, StdoutOutput));
             }
         }
     }
 }
 
-#endif // QT_CONFIG(process)
-
 // ----------------------- Entrypoint -----------------------
 
 int main(int argc, char **argv)
 {
-#if !QT_CONFIG(process)
-    return 0;
-#else
     std::vector<const char*> args(argv, argv + argc);
 
     static auto kRebaseArgument = "--rebase";
@@ -1264,6 +1270,5 @@ int main(int argc, char **argv)
     }
 
     return result;
-#endif
 }
 
