@@ -26,19 +26,18 @@
 **
 ****************************************************************************/
 
-#include <QtWidgets/QOpenGLWidget>
+#include <QtOpenGLWidgets/QOpenGLWidget>
 #include <QtGui/QOpenGLFunctions>
 #include <QtGui/QPainter>
 #include <QtGui/QScreen>
 #include <QtGui/QStaticText>
-#include <QtWidgets/QDesktopWidget>
 #include <QtWidgets/QGraphicsView>
 #include <QtWidgets/QGraphicsScene>
 #include <QtWidgets/QGraphicsRectItem>
 #include <QtWidgets/QVBoxLayout>
 #include <QtWidgets/QPushButton>
 #include <QtWidgets/QStackedWidget>
-#include <QtTest/QtTest>
+#include <QTest>
 #include <QSignalSpy>
 #include <private/qguiapplication_p.h>
 #include <private/qstatictext_p.h>
@@ -49,9 +48,6 @@ class tst_QOpenGLWidget : public QObject
 {
     Q_OBJECT
 
-public:
-    static void initMain() { QCoreApplication::setAttribute(Qt::AA_DisableHighDpiScaling); }
-
 private slots:
     void initTestCase();
     void create();
@@ -70,6 +66,7 @@ private slots:
     void stackWidgetOpaqueChildIsVisible();
     void offscreen();
     void offscreenThenOnscreen();
+    void paintWhileHidden();
 
 #ifdef QT_BUILD_INTERNAL
     void staticTextDanglingPointer();
@@ -365,6 +362,16 @@ void tst_QOpenGLWidget::asViewport()
     widget.show();
     QVERIFY(QTest::qWaitForWindowExposed(&widget));
 
+    if (QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation)) {
+        // On some platforms (macOS), the palette will be different depending on if a
+        // window is active or not. And because of that, the whole window will be
+        // repainted when going from Inactive to Active. So wait for the window to be
+        // active before we continue, so the activation doesn't happen at a random
+        // time below. And call processEvents to have the paint events delivered right away.
+        QVERIFY(QTest::qWaitForWindowActive(&widget));
+        qApp->processEvents();
+    }
+
     QVERIFY(view->paintCount() > 0);
     view->resetPaintCount();
 
@@ -388,6 +395,9 @@ public:
 
 void tst_QOpenGLWidget::requestUpdate()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+
     PaintCountWidget w;
     w.resize(640, 480);
     w.show();
@@ -495,23 +505,19 @@ static inline QString msgRgbMismatch(unsigned actual, unsigned expected)
 
 static QPixmap grabWidgetWithoutRepaint(const QWidget *widget, QRect clipArea)
 {
-    const QWidget *targetWidget = widget;
+    const QWindow *window = widget->window()->windowHandle();
+    Q_ASSERT(window);
+    WId windowId = window->winId();
+
 #ifdef Q_OS_WIN
     // OpenGL content is not properly grabbed on Windows when passing a top level widget window,
     // because GDI functions can't grab OpenGL layer content.
     // Instead the whole screen should be captured, with an adjusted clip area, which contains
     // the final composited content.
-    QDesktopWidget *desktopWidget = QApplication::desktop();
-    const QWidget *mainScreenWidget = desktopWidget->screen();
-    targetWidget = mainScreenWidget;
+    windowId = 0;
     clipArea = QRect(widget->mapToGlobal(clipArea.topLeft()),
                      widget->mapToGlobal(clipArea.bottomRight()));
 #endif
-
-    const QWindow *window = targetWidget->window()->windowHandle();
-    Q_ASSERT(window);
-    WId windowId = window->winId();
-
     QScreen *screen = window->screen();
     Q_ASSERT(screen);
 
@@ -576,10 +582,14 @@ bool verifyColor(const QWidget *widget, const QRect &clipArea, const QColor &col
 
 void tst_QOpenGLWidget::stackWidgetOpaqueChildIsVisible()
 {
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     QSKIP("QScreen::grabWindow() doesn't work properly on OSX HighDPI screen: QTBUG-46803");
     return;
 #endif
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+    if (QGuiApplication::platformName().startsWith(QLatin1String("offscreen"), Qt::CaseInsensitive))
+        QSKIP("Offscreen: This fails.");
 
     QStackedWidget stack;
 
@@ -685,6 +695,30 @@ void tst_QOpenGLWidget::offscreenThenOnscreen()
     QVERIFY(image.pixel(30, 40) == qRgb(0, 0, 255));
 }
 
+void tst_QOpenGLWidget::paintWhileHidden()
+{
+    QScopedPointer<QWidget> tlw(new QWidget);
+    tlw->resize(640, 480);
+
+    ClearWidget *w = new ClearWidget(0, 640, 480);
+    w->setParent(tlw.data());
+    w->setClearColor(0, 0, 1);
+
+    tlw->show();
+    QVERIFY(QTest::qWaitForWindowExposed(tlw.data()));
+
+    // QTBUG-101620: Now make visible=false and call update and see if we get to
+    // paintEvent/paintGL eventually, to ensure the updating of the texture is
+    // not optimized permanently away even though there is no composition
+    // on-screen at the point when update() is called.
+
+    w->setVisible(false);
+    w->m_paintCalled = 0;
+    w->update();
+    w->setVisible(true);
+    QTRY_VERIFY(w->m_paintCalled > 0);
+}
+
 class StaticTextPainterWidget : public QOpenGLWidget
 {
 public:
@@ -693,7 +727,7 @@ public:
     {
     }
 
-    void paintEvent(QPaintEvent *)
+    void paintEvent(QPaintEvent *) override
     {
         QPainter p(this);
         text.setText(QStringLiteral("test"));
