@@ -1,6 +1,6 @@
 #############################################################################
 ##
-## Copyright (C) 2018 The Qt Company Ltd.
+## Copyright (C) 2021 The Qt Company Ltd.
 ## Contact: https://www.qt.io/licensing/
 ##
 ## This file is part of the plugins of the Qt Toolkit.
@@ -98,7 +98,7 @@ _qt_library_map = [
     LibraryMapping(
         "application-lib", "Qt6", "Qt::AppManApplication", extra=["COMPONENTS", "AppManApplication"]
     ),
-    LibraryMapping("axbase", "Qt6", "Qt::AxBase", extra=["COMPONENTS", "AxBase"]),
+    LibraryMapping("axbase", "Qt6", "Qt::AxBasePrivate", extra=["COMPONENTS", "AxBasePrivate"]),
     LibraryMapping("axcontainer", "Qt6", "Qt::AxContainer", extra=["COMPONENTS", "AxContainer"]),
     LibraryMapping("axserver", "Qt6", "Qt::AxServer", extra=["COMPONENTS", "AxServer"]),
     LibraryMapping("bluetooth", "Qt6", "Qt::Bluetooth", extra=["COMPONENTS", "Bluetooth"]),
@@ -123,20 +123,20 @@ _qt_library_map = [
     LibraryMapping(
         "designercomponents",
         "Qt6",
-        "Qt::DesignerComponents",
-        extra=["COMPONENTS", "DesignerComponents"],
+        "Qt::DesignerComponentsPrivate",
+        extra=["COMPONENTS", "DesignerComponentsPrivate"],
     ),
     LibraryMapping(
         "devicediscovery",
         "Qt6",
-        "Qt::DeviceDiscoverySupport",
-        extra=["COMPONENTS", "DeviceDiscoverySupport"],
+        "Qt::DeviceDiscoverySupportPrivate",
+        extra=["COMPONENTS", "DeviceDiscoverySupportPrivate"],
     ),
     LibraryMapping(
         "devicediscovery_support",
         "Qt6",
-        "Qt::DeviceDiscoverySupport",
-        extra=["COMPONENTS", "DeviceDiscoverySupport"],
+        "Qt::DeviceDiscoverySupportPrivate",
+        extra=["COMPONENTS", "DeviceDiscoverySupportPrivate"],
     ),
     LibraryMapping("edid", "Qt6", "Qt::EdidSupport", extra=["COMPONENTS", "EdidSupport"]),
     LibraryMapping("edid_support", "Qt6", "Qt::EdidSupport", extra=["COMPONENTS", "EdidSupport"]),
@@ -144,17 +144,20 @@ _qt_library_map = [
     LibraryMapping(
         "eglfsdeviceintegration",
         "Qt6",
-        "Qt::EglFSDeviceIntegration",
-        extra=["COMPONENTS", "EglFSDeviceIntegration"],
+        "Qt::EglFSDeviceIntegrationPrivate",
+        extra=["COMPONENTS", "EglFSDeviceIntegrationPrivate"],
     ),
     LibraryMapping(
-        "eglfs_kms_support", "Qt6", "Qt::EglFsKmsSupport", extra=["COMPONENTS", "EglFsKmsSupport"]
+        "eglfs_kms_support",
+        "Qt6",
+        "Qt::EglFsKmsSupportPrivate",
+        extra=["COMPONENTS", "EglFsKmsSupportPrivate"],
     ),
     LibraryMapping(
         "eglfs_kms_gbm_support",
         "Qt6",
-        "Qt::EglFsKmsGbmSupport",
-        extra=["COMPONENTS", "EglFsKmsGbmSupport"],
+        "Qt::EglFsKmsGbmSupportPrivate",
+        extra=["COMPONENTS", "EglFsKmsGbmSupportPrivate"],
     ),
     LibraryMapping("egl_support", "Qt6", "Qt::EglSupport", extra=["COMPONENTS", "EglSupport"]),
     # enginio: Not needed in Qt6!
@@ -170,8 +173,12 @@ _qt_library_map = [
         "Qt::EventDispatcherSupport",
         extra=["COMPONENTS", "EventDispatcherSupport"],
     ),
-    LibraryMapping("fbconvenience", "Qt6", "Qt::FbSupport", extra=["COMPONENTS", "FbSupport"]),
-    LibraryMapping("fb_support", "Qt6", "Qt::FbSupport", extra=["COMPONENTS", "FbSupport"]),
+    LibraryMapping(
+        "fbconvenience", "Qt6", "Qt::FbSupportPrivate", extra=["COMPONENTS", "FbSupportPrivate"]
+    ),
+    LibraryMapping(
+        "fb_support", "Qt6", "Qt::FbSupportPrivate", extra=["COMPONENTS", "FbSupportPrivate"]
+    ),
     LibraryMapping(
         "fontdatabase_support",
         "Qt6",
@@ -179,6 +186,7 @@ _qt_library_map = [
         extra=["COMPONENTS", "FontDatabaseSupport"],
     ),
     LibraryMapping("gamepad", "Qt6", "Qt::Gamepad", extra=["COMPONENTS", "Gamepad"]),
+    LibraryMapping("geniviextras", "Qt6", "Qt::GeniviExtras", extra=["COMPONENTS", "GeniviExtras"]),
     LibraryMapping(
         "global", "Qt6", "Qt::Core", extra=["COMPONENTS", "Core"]
     ),  # manually added special case
@@ -191,19 +199,31 @@ _qt_library_map = [
     LibraryMapping(
         "hunspellinputmethod",
         "Qt6",
-        "Qt::HunspellInputMethod",
-        extra=["COMPONENTS", "HunspellInputMethod"],
+        "Qt::HunspellInputMethodPrivate",
+        extra=["COMPONENTS", "HunspellInputMethodPrivate"],
     ),
-    LibraryMapping("input", "Qt6", "Qt::InputSupport", extra=["COMPONENTS", "InputSupport"]),
     LibraryMapping(
-        "input_support", "Qt6", "Qt::InputSupport", extra=["COMPONENTS", "InputSupport"]
+        "input", "Qt6", "Qt::InputSupportPrivate", extra=["COMPONENTS", "InputSupportPrivate"]
+    ),
+    LibraryMapping(
+        "input_support",
+        "Qt6",
+        "Qt::InputSupportPrivate",
+        extra=["COMPONENTS", "InputSupportPrivate"],
     ),
     LibraryMapping(
         "installer-lib", "Qt6", "Qt::AppManInstaller", extra=["COMPONENTS", "AppManInstaller"]
     ),
+    LibraryMapping("ivi", "Qt6", "Qt::Ivi", extra=["COMPONENTS", "Ivi"]),
+    LibraryMapping("ivicore", "Qt6", "Qt::IviCore", extra=["COMPONENTS", "IviCore"]),
+    LibraryMapping("ivimedia", "Qt6", "Qt::IviMedia", extra=["COMPONENTS", "IviMedia"]),
     LibraryMapping("knx", "Qt6", "Qt::Knx", extra=["COMPONENTS", "Knx"]),
-    LibraryMapping("kmsconvenience", "Qt6", "Qt::KmsSupport", extra=["COMPONENTS", "KmsSupport"]),
-    LibraryMapping("kms_support", "Qt6", "Qt::KmsSupport", extra=["COMPONENTS", "KmsSupport"]),
+    LibraryMapping(
+        "kmsconvenience", "Qt6", "Qt::KmsSupportPrivate", extra=["COMPONENTS", "KmsSupportPrivate"]
+    ),
+    LibraryMapping(
+        "kms_support", "Qt6", "Qt::KmsSupportPrivate", extra=["COMPONENTS", "KmsSupportPrivate"]
+    ),
     LibraryMapping(
         "launcher-lib", "Qt6", "Qt::AppManLauncher", extra=["COMPONENTS", "AppManLauncher"]
     ),
@@ -247,10 +267,16 @@ _qt_library_map = [
         "package-lib", "Qt6", "Qt::AppManPackage", extra=["COMPONENTS", "AppManPackage"]
     ),
     LibraryMapping(
-        "packetprotocol", "Qt6", "Qt::PacketProtocol", extra=["COMPONENTS", "PacketProtocol"]
+        "packetprotocol",
+        "Qt6",
+        "Qt::PacketProtocolPrivate",
+        extra=["COMPONENTS", "PacketProtocolPrivate"],
     ),
     LibraryMapping(
-        "particles", "Qt6", "Qt::QuickParticles", extra=["COMPONENTS", "QuickParticles"]
+        "particles",
+        "Qt6",
+        "Qt::QuickParticlesPrivate",
+        extra=["COMPONENTS", "QuickParticlesPrivate"],
     ),
     LibraryMapping(
         "plugin-interfaces",
@@ -264,18 +290,24 @@ _qt_library_map = [
     ),
     LibraryMapping("printsupport", "Qt6", "Qt::PrintSupport", extra=["COMPONENTS", "PrintSupport"]),
     LibraryMapping("purchasing", "Qt6", "Qt::Purchasing", extra=["COMPONENTS", "Purchasing"]),
-    LibraryMapping("qmldebug", "Qt6", "Qt::QmlDebug", extra=["COMPONENTS", "QmlDebug"]),
-    LibraryMapping("qmldevtools", "Qt6", "Qt::QmlDevTools", extra=["COMPONENTS", "QmlDevTools"]),
-    LibraryMapping("qmlcompiler", "Qt6", "Qt::QmlCompiler", extra=["COMPONENTS", "QmlCompiler"]),
+    LibraryMapping(
+        "qmldebug", "Qt6", "Qt::QmlDebugPrivate", extra=["COMPONENTS", "QmlDebugPrivate"]
+    ),
+    LibraryMapping(
+        "qmldevtools", "Qt6", "Qt::QmlDevToolsPrivate", extra=["COMPONENTS", "QmlDevToolsPrivate"]
+    ),
+    LibraryMapping(
+        "qmlcompiler", "Qt6", "Qt::QmlCompilerPrivate", extra=["COMPONENTS", "QmlCompilerPrivate"]
+    ),
     LibraryMapping("qml", "Qt6", "Qt::Qml", extra=["COMPONENTS", "Qml"]),
-    LibraryMapping("qmldom", "Qt6", "Qt::QmlDom", extra=["COMPONENTS", "QmlDom"]),
+    LibraryMapping("qmldom", "Qt6", "Qt::QmlDomPrivate", extra=["COMPONENTS", "QmlDomPrivate"]),
     LibraryMapping("qmlmodels", "Qt6", "Qt::QmlModels", extra=["COMPONENTS", "QmlModels"]),
     LibraryMapping("qmltest", "Qt6", "Qt::QuickTest", extra=["COMPONENTS", "QuickTest"]),
     LibraryMapping(
         "qtmultimediaquicktools",
         "Qt6",
-        "Qt::MultimediaQuick",
-        extra=["COMPONENTS", "MultimediaQuick"],
+        "Qt::MultimediaQuickPrivate",
+        extra=["COMPONENTS", "MultimediaQuickPrivate"],
     ),
     LibraryMapping(
         "quick3dassetimport",
@@ -305,7 +337,9 @@ _qt_library_map = [
         extra=["COMPONENTS", "QuickControls2Impl"],
     ),
     LibraryMapping("quick", "Qt6", "Qt::Quick", extra=["COMPONENTS", "Quick"]),
-    LibraryMapping("quickshapes", "Qt6", "Qt::QuickShapes", extra=["COMPONENTS", "QuickShapes"]),
+    LibraryMapping(
+        "quickshapes", "Qt6", "Qt::QuickShapesPrivate", extra=["COMPONENTS", "QuickShapesPrivate"]
+    ),
     LibraryMapping(
         "quicktemplates2", "Qt6", "Qt::QuickTemplates2", extra=["COMPONENTS", "QuickTemplates2"]
     ),
@@ -360,7 +394,9 @@ _qt_library_map = [
     LibraryMapping("window-lib", "Qt6", "Qt::AppManWindow", extra=["COMPONENTS", "AppManWindow"]),
     LibraryMapping("winextras", "Qt6", "Qt::WinExtras", extra=["COMPONENTS", "WinExtras"]),
     LibraryMapping("x11extras", "Qt6", "Qt::X11Extras", extra=["COMPONENTS", "X11Extras"]),
-    LibraryMapping("xcb_qpa_lib", "Qt6", "Qt::XcbQpa", extra=["COMPONENTS", "XcbQpa"]),
+    LibraryMapping(
+        "xcb_qpa_lib", "Qt6", "Qt::XcbQpaPrivate", extra=["COMPONENTS", "XcbQpaPrivate"]
+    ),
     LibraryMapping(
         "xkbcommon_support", "Qt6", "Qt::XkbCommonSupport", extra=["COMPONENTS", "XkbCommonSupport"]
     ),
@@ -370,7 +406,10 @@ _qt_library_map = [
         "qmlworkerscript", "Qt6", "Qt::QmlWorkerScript", extra=["COMPONENTS", "QmlWorkerScript"]
     ),
     LibraryMapping(
-        "quickparticles", "Qt6", "Qt::QuickParticles", extra=["COMPONENTS", "QuickParticles"]
+        "quickparticles",
+        "Qt6",
+        "Qt::QuickParticlesPrivate",
+        extra=["COMPONENTS", "QuickParticlesPrivate"],
     ),
     LibraryMapping(
         "linuxofono_support",
@@ -398,6 +437,9 @@ _qt_library_map = [
 _library_map = [
     # 3rd party:
     LibraryMapping("atspi", "ATSPI2", "PkgConfig::ATSPI2"),
+    LibraryMapping(
+        "backtrace", "WrapBacktrace", "WrapBacktrace::WrapBacktrace", emit_if="config.unix"
+    ),
     LibraryMapping("bluez", "BlueZ", "PkgConfig::BlueZ"),
     LibraryMapping("brotli", "WrapBrotli", "WrapBrotli::WrapBrotliDec"),
     LibraryMapping("corewlan", None, None),
@@ -408,6 +450,7 @@ _library_map = [
     LibraryMapping(
         "doubleconversion", "WrapDoubleConversion", "WrapDoubleConversion::WrapDoubleConversion"
     ),
+    LibraryMapping("dlt", "DLT", "DLT::DLT"),
     LibraryMapping("drm", "Libdrm", "Libdrm::Libdrm"),
     LibraryMapping("egl", "EGL", "EGL::EGL"),
     LibraryMapping("flite", "Flite", "Flite::Flite"),
@@ -451,6 +494,7 @@ _library_map = [
     LibraryMapping("librt", "WrapRt", "WrapRt::WrapRt"),
     LibraryMapping("libudev", "Libudev", "PkgConfig::Libudev"),
     LibraryMapping("lttng-ust", "LTTngUST", "LTTng::UST", resultVariable="LTTNGUST"),
+    LibraryMapping("libmd4c", "WrapMd4c", "WrapMd4c::WrapMd4c", is_bundled_with_qt=True),
     LibraryMapping("mtdev", "Mtdev", "PkgConfig::Mtdev"),
     LibraryMapping("mysql", "MySQL", "MySQL::MySQL"),
     LibraryMapping("odbc", "ODBC", "ODBC::ODBC"),
@@ -489,10 +533,13 @@ _library_map = [
     LibraryMapping("sqlite2", None, None),  # No more sqlite2 support in Qt6!
     LibraryMapping("sqlite3", "SQLite3", "SQLite::SQLite3"),
     LibraryMapping("sqlite", "SQLite3", "SQLite::SQLite3"),
+    LibraryMapping(
+        "taglib", "WrapTagLib", "WrapTagLib::WrapTagLib", is_bundled_with_qt=True
+    ),  # used in qtivi
     LibraryMapping("tslib", "Tslib", "PkgConfig::Tslib"),
     LibraryMapping("udev", "Libudev", "PkgConfig::Libudev"),
     LibraryMapping("udev", "Libudev", "PkgConfig::Libudev"),  # see also libudev!
-    LibraryMapping("vulkan", "Vulkan", "Vulkan::Vulkan"),
+    LibraryMapping("vulkan", "WrapVulkanHeaders", "WrapVulkanHeaders::WrapVulkanHeaders"),
     LibraryMapping("wayland_server", "Wayland", "Wayland::Server"),  # used in qtbase/src/gui
     LibraryMapping("wayland-server", "Wayland", "Wayland::Server"),  # used in qtwayland
     LibraryMapping("wayland-client", "Wayland", "Wayland::Client"),
@@ -595,7 +642,7 @@ _library_map = [
     LibraryMapping("xlib", "X11", "X11::X11"),
     LibraryMapping("xrender", "XRender", "PkgConfig::XRender", extra=["0.6"]),
     LibraryMapping("zlib", "WrapZLIB", "WrapZLIB::WrapZLIB", extra=["1.0.8"]),
-    LibraryMapping("zstd", "ZSTD", "ZSTD::ZSTD", extra=["1.3"]),
+    LibraryMapping("zstd", "WrapZSTD", "WrapZSTD::WrapZSTD", extra=["1.3"]),
     LibraryMapping("tiff", "TIFF", "TIFF::TIFF"),
     LibraryMapping("webp", "WrapWebP", "WrapWebP::WrapWebP"),
     LibraryMapping("jasper", "WrapJasper", "WrapJasper::WrapJasper"),
@@ -624,14 +671,18 @@ _library_map = [
 
 
 def _adjust_library_map():
-    # Assign a Linux condition on all x and wayland related packages.
+    # Assign a Linux condition on all wayland related packages.
+    # Assign platforms that have X11 condition on all X11 related packages.
     # We don't want to get pages of package not found messages on
     # Windows and macOS, and this also improves configure time on
     # those platforms.
-    linux_package_prefixes = ["xcb", "x11", "xkb", "xrender", "xlib", "wayland"]
+    linux_package_prefixes = ["wayland"]
+    x11_package_prefixes = ["xcb", "x11", "xkb", "xrender", "xlib"]
     for i, _ in enumerate(_library_map):
         if any([_library_map[i].soName.startswith(p) for p in linux_package_prefixes]):
             _library_map[i].emit_if = "config.linux"
+        if any([_library_map[i].soName.startswith(p) for p in x11_package_prefixes]):
+            _library_map[i].emit_if = "X11_SUPPORTED"
 
 
 _adjust_library_map()
@@ -669,7 +720,7 @@ def find_library_info_for_target(targetName: str) -> typing.Optional[LibraryMapp
 
 # For a given qmake library (e.g. 'openssl_headers'), check whether this is a fake library used
 # for the /nolink annotation, and return the actual annotated qmake library ('openssl/nolink').
-def find_annotated_qmake_lib_name(lib : str) -> str:
+def find_annotated_qmake_lib_name(lib: str) -> str:
     for entry in _library_map:
         if entry.no_link_so_name == lib:
             return entry.soName + "/nolink"
@@ -710,7 +761,6 @@ platform_mapping = {
     "hpux": "HPUX",
     "nacl": "NACL",
     "android": "ANDROID",
-    "android-embedded": "ANDROID_EMBEDDED",
     "uikit": "UIKIT",
     "tvos": "TVOS",
     "watchos": "WATCHOS",
